
#
# Perl Module for AccMap
#
# Cared for by ewan birney <birney@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package AccMap;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );



my %fields = (
	      id => undef, # hash of id to acc
	      acc => undef # hash of acc to id
    #Insert field names here as field => undef,
);


sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = {
	'_permitted' => \%fields,
	%fields, };
    $self->{'id'} = {};
    $self->{'acc'} = {};
    bless $self, $class;
    return $self;
}


sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || carp "$self is not an object - can't therefore find a member!";
    my $name = $AUTOLOAD;
    $name =~ /::DESTROY/ && return;
    $name =~ s/.*://;
    unless (exists $self->{'_permitted'}->{$name} ) {
	carp "In type $type, can't access $name - probably passed a wrong variable into AccMap";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name};
    }
}

sub idfromacc {
    my $self = shift;
    my $acc  = shift;
    return $self->{'acc'}->{$acc};
}

sub accfromid {
    my $self = shift;
    my $id  = shift;
    return $self->{'id'}->{$id};
}

sub read2col {
    my $self = shift;
    my $f = shift;
    my ($acc,$id);

    while(<$f>) {
	($id,$acc) = split;
	$self->{'id'}->{$id} = $acc;
	$self->{'acc'}->{$acc} = $id;
    }

}

sub readspem {
    my $self = shift;
    my $f = shift;
    my ($acc,$id);

    while(<$f>) {
	/^ID\s+(\w+)/ || next;
	$id = $1;
	while(<$f>) {
	    /^AC\s+(\w+)/ || next;
	    $acc=$1;
	   # print STDERR "Got $id $acc\n";
	    $self->{'id'}->{$id} = $acc;
	    $self->{'acc'}->{$acc} = $id;
	    last;
	}
    }
		
}



1;  # says use was ok
__END__

=head1 NAME

AccMap

=head1 DESCRIPTION

Really really simple double hash to provide id to acc mapping.
Useful when doing large scale database manipulations. *don't* use
it for individual sequences...

=head1 AUTHOR

B<ewan birney> Email birney@sanger.ac.uk

=over

=item idfromacc

returns your id from accession number

=item accfromid

returns accession number from id

=item read2col

read a file with <id> <acc> as a two column file

=item readspem

reads a swissprot/embl .dat formatted file

