
package AceParse;

use strict;
use Carp;
#use humpubace qw( timestamp );
use Exporter ();
use vars qw( @ISA @EXPORT_OK );
@ISA = qw( Exporter );
@EXPORT_OK = qw( aceParse aceQuote );

# Generates an acedb timestamp
sub timestamp (;$) {
    my $time = shift || time;

    # Get time info
    my ($sec, $min, $hour, $mday, $mon, $year) = (localtime($time))[0..5];

    # Change numbers to double-digit format
    ($mon, $mday, $hour, $min, $sec) = ('00'..'59')[($mon + 1), $mday, $hour, $min, $sec];

    # Make year
    $year += 1900;

    # Translate the effective userID into a user name
    my $id = (getpwuid($>))[0];

    return "$year-$mon-${mday}_$hour:$min:${sec}_$id";
}

# Prototype forces scalar context of second
# argument, so can say _getIndex -2, @row
sub _getIndex ($$) {
    my ($index, $length) = @_;
    # Can give -1 for last element etc...
    if ($index < 0 ) {
	$index = $length + $index;
	# If negative index takes us off the front
	# of the array, then return first element
	if ($index < 0) { $index = 0 }
    }
    return $index;
}

sub new {
    my $pkg = shift;
    return bless [], $pkg;
}

sub aceTableFromStream {
    my( $pkg, $fh, $keep_comments ) = @_;

    # Ace file objects are separated one or more blank lines
    local $/ = '';

    # Return undef if nothing to read
    my $aceBlock = <$fh>;
    return unless defined( $aceBlock );

    # Make new table object from array of unparsed ace lines
    $pkg->_tableFromAceLines( [split /\n/, $aceBlock], $keep_comments );
}

sub aceTable {
    my( $pkg, $aceRef, $keep_comments ) = @_;
    my $lines;
    
    if ( ref( $aceRef ) eq 'SCALAR' ) {
	$lines = [ split /\n/, $$aceRef ];
    } elsif ( ref( $aceRef ) eq 'ARRAY' ) {
	$lines = $aceRef;
    } else {
	confess "Error: aceTable: argument must be ref to either SCALAR or ARRAY";
    }

    # Make new table object from array of unparsed ace lines
    $pkg->_tableFromAceLines( $lines );
}

# Private funcion which deals with backslash continued line
sub _tableFromAceLines {
    my( $pkg, $lines, $keep_comments ) = @_;

    # Remove blank lines from start of array
    while ( $lines->[0] =~ /^\s+$/ ) {
	shift @$lines;
    }

    # Make a new table object
    my $table = $pkg->new();

    while (my $line = shift @$lines) {
	chomp $line;
	last unless length($line);
	# Loop to cope with '\' continued lines
	while ($line =~ s/\\$//) {
            my $next = shift @$lines
                or confess "No more lines in object after '$line'";
	    $line .= $next;
	    chomp $line;
	}
        # Loop to cope with unbalanced double quotes
        while ($line =~ tr/"/"/ % 2) {
            my $next = shift @$lines
                or confess "No more lines in object after '$line'";
	    $line .= $next;
            chomp $line;
        }
	# parse into ace tags and store
	$table->addRow( aceParse( $line, $keep_comments ) );
    }
    return $table;
}

sub class {
    my $table = shift;

    unless ($table->[0][1] eq ':') {
	return;
#  	unless ($table->row(0) == 2) {
#  	    confess "First line of table is [", $table->row(0), "]\n",
#  	    "Can't use class method on partial ace object\n";
#  	}
    }
    if (@_) {
	$table->[0][0] = shift;
	return 1;
    } else {
	return $table->[0][0];
    }
}

sub name {
    my $table = shift;

    my $index = 2;

    unless ($table->[0][1] eq ':') {
	return;
#  	$index = 1;
#  	unless ($table->row(0) == 2) {
#  	    confess "First line of table is [", $table->row(0), "]\n",
#  	    "Can't use name method on partial ace object\n";
#  	}
    }

    if (@_) {
	$table->[0][$index] = shift;
	return 1;
    } else {
	return $table->[0][$index];
    }
}

sub addRow {
    my $table = shift;
    
    push( @$table, [@_] );

}

sub cell {
    my $table = shift;
    my $row = shift;
    my $column = shift;

    my $ts; # To flag if line is timestamped

    return unless defined $column;

    # Can provide negative numbers: -1 is last row,
    # -2 is next to last, etc...
    $row = _getIndex $row, @{$table};

    # Deal with timestamp contaminated lines
    # by transparently adjusting column index
    my $count = scalar @{$table->[$row]};
    my $d = $table->[$row][1];
    if ($d and $d eq '-O') {
	$ts = 1;
	if ($count % 3) {
	    confess "Error: Wrong number of elements [$count] in timestamp line: ",
	    (join '|', $table->[$row]);
	} else {
	    # Deal with negative index values
	    $column = _getIndex $column, $count / 3;
	    # Adjust the numbering
	    $column = (($column + 1) * 3) - 3;
	}
    } else {
	$column = _getIndex $column, @{$table->[$row]};
    }
	    
    if (@_) {
	$table->[$row][$column] = shift;
	if ($ts) {
	    # Change timestamp as well
	    $table->[$row][($column + 2)] = timestamp();
	}
	return 1;
    } else {
	return $table->[$row][$column];
    }
}

sub row {
    my $table = shift;
    my $row = shift;

    if (defined $row) { # $row may be 0
	# Asking about a particular line

	# Generate index for negative numbers
	$row = _getIndex $row, @{$table};

	if (wantarray) {
	    return @{$table->[$row]};
	} else {
	    my $count = scalar @{$table->[$row]};

	    # Report number of elements, ignoring
	    # timestamp objects if present
	    if ($table->[$row][1] eq '-O') {
		if ($count % 3) {
		    confess "Error: Wrong number of elements in timestamp line: ",
		    aceQuote( $table->row($row) );
		} else {
		    return $count / 3;
		}
	    } else {
		return $count;
	    }
	}
    } else {
	# Are asking about the whole table?
	if (wantarray) {
	    return @$table;
	} else {
	    return scalar @$table;
	}
    }
}

sub rowIndex {
    my $table = shift;

    if (wantarray) {
	return (0..$#{$table});
    } else {
	return $#{$table};
    }
}

sub shiftRow {
    my $table = shift;

    # return if the table is empty
    my $l = shift( @$table ) or return;

    # else return a real array
    return @$l;

}

sub aceQuote (@) {
    my @line = @_;
    my $start; # First element of line to examine
    
    # From rd :-
    #
    # We escape:
    #         \ " / % ; \t \n
    # The last two are the tab and newline character.  The newline character
    # is treated differently from all the rest - it is converted into the
    # four character string "\n\n", i.e. backslash-en-backslash-en.  This
    # looks odd, but there is a note in the subroutine that it is what is
    # needed to make things work, and I guess that is true!  However, it
    # may be that something further along in our code strips off one pair
    # - best to check for yourself.

    # The first element of an aceline is never quoted,
    # but will also skip second if the line is a -D or -R
    # command
    $line[0] =~ /^-[DR]$/ ? $start = 2 : $start = 1;

    # Changing $_ modifies @line in-place
    foreach (@line[$start..$#line]) {
	next if /^\/{2}/;    # Don't quote comments
	next if /^-O$/;      # Don't quote -O (timestamps)
	next if /^[\.\d]+$/; # Don't quote numbers
	next if /^:$/;       # Don't quote elements which are just :

	# Escape quotes, back and forward slashes,
	# % signs, and semi-colons.
	s|([/"%;\\])|\\$1|g;

	# Escape newlines and tabs
	s|\n|\\n|g;
	s|\n|\\t|g;

	# Quote if tag unless it is entirely word characters
        unless (/^\w+$/) {
	    $_ = "\"$_\"";
	}
    }

    # Return a fully formatted line
    return (join ' ', @line) . "\n";
}

sub print {
    my $table = shift;
    my $fh = shift || \*STDOUT;
    my $lineCount = $table->row;

    # Only print if the table has lines
    
    if ($lineCount) {
        foreach my $row ($table->row()) {
	    print $fh aceQuote( @$row );
        }
        print $fh "\n";
        return $lineCount;
    } else {
        return;
    }
}

sub aceParse ($) {
    local $_ = shift;
    my $keep_comments = shift;
    
    my( @tags,  # Array of parsed tags to return
       $comment # Comments to restore to output
       );
    
    # Skip blank lines
    return () if /^$/;
    
    # Save line for reporting errors
    my $saveLine = $_;
    
    # Save comment part of line if $keep_comments is set
    if ($keep_comments) {
        s/^(.*?)(\/{2}.*)/$1/;
	$comment = $2;
    }

    while (length) {
	my $q = 0; # Set inside quoted text
	my $field; # Element to add to tags array
	for (;;) {
	    # Are inside single " ?
	    if ( $q ) {
                # Are at end of quoted text
                if (s/^"(\s+|\Z)//) {
                    $q = 0;
                    last;
                # Save " from \" at start of string
		} elsif (s/^\\(")//) {
		    $field .= $1;
                # Or everything until next \ or "
                } elsif (s/^(.[^"\\]*)//) {
                    $field .= $1;
                # Or something is wrong
                } else {
	            confess "Unbalanced \" in line:\n$saveLine\n";
                }
	    } else {
		# Are at start of new quoted string
		if (s/^"//) {
                    $field = ''; # Field is now defined, which produces
                                 # the correct behaviour for empty tags
		    $q = 1;
                # Or have an unquoted string
		} elsif (s/^(\S+)\s*//) {
                    $field = $1;
		    last;
                # Or have spaces on start of line
		} else {
                    s/^\s+//;
                    # Exit infinte loop if nothing left
	            last unless length;
		}
	    }
	}
        
        if ($field) {
	    # Change \t and \n to unescaped characters
	    $field =~ s|\\t|\t|g;
	    $field =~ s|\\n|\n|g;
            # Unescape backslashed "/", "\", "%" and ";" charachers
	    $field =~ s|\\([/%;\\])|$1|g;
        }

	# Must use defined, since element may be "0"
	push( @tags, $field ) if defined $field;
    }
    push( @tags, $comment ) if $comment;	
    return( @tags );
}

1;


__END__

=head1 NAME - AceParse.pm

=head1 DESCRIPTION

Module for parsing ace files.  It allows you to treat ACeDB ".ace"
flat files (ace files) as a series of table objects, which are arrays
of arrays.  The aceParse and aceQuote functions can be imported.

This module was written to facilitate writing filters on dumped acedb
databases, for cleaning up data.

=head1 SYNOPSIS

	use AceParse qw( aceParse aceQuote );

	# Filter ace files given on command line
	while (my $table = AceParse->aceTableFromStream(\*ARGV)) {
	    my $class = $table->class;
	    my $name = $table->name;

	    if ($class eq 'Sequence') {
		foreach $i ($table->rowIndex) {
		    if ($table->cell( $i, 0 ) eq 'Homol') {
			# Don't print out incomplete Homol lines
			next unless $table->row($i) < 9;
		    }
		    print aceQuote( $table->row($i));
		}
		print "\n";
	    } else {
		$table->print;
	    }
	}

=head1 FUNCTIONS

=over 4

=item aceParse

	@tagArray = aceParse $line;

Parses a line of an ace file, dealing correctly with quotes.
B<aceParse> confesses if it finds unbalanced quotes on the line.  This is
different to the behaviour of ACeDB itself, which allows unbalanced
quotes.  Note that a line in an ace file may be backslash continued,
so you can't just call this on every line of an ace file, unless you
know that your file won't contain backslash continued lines.
B<aceTable> and B<aceTableFromStream> both deal with backslash
continued lines.

=item aceQuote

	$aceLine = aceQuote( @ace );
	print aceQuote @ace;

Takes an array, where each element is an acedb tag, and correctly
quotes it for printing to an ace file.

=item aceTable

	$table = AceParse->aceTable( \$aceBlock );
	$table = AceParse->aceTable( \@aceBlockLines );

Takes an acefile block, and returns a new table object, which is an
array of arrays.  An acefile block is a block of text in the ace
file, which is separated by one or more blank lines from the next
block of text.  B<aceTable> takes a referance to either a scalar or an
array, and returns a table object.  It copes with backslash continued
lines, and correctly unquotes text inside fields.

=item aceTableFromStream

	$table = AceParse->tableFromStream(\*ARGV);

Reads the next ace object from a file handle by reading in the next
acefile block, and parsing it correctly in the same way that
B<aceTable> does.

=item class and name

	$class = $table->class();
	$table->class( 'Sequence' );

	$name = $table->name();
	$table->name( 'bK390B3_copy' );

Give or set the class or name of the table object.  These functions
only operate if the second element of the first row of the table is
":", which is always true if the ace block is generated by ACeDB.
They return undef on failure.

=item new

	$table = AceParse->new();

Creates a new empty table object.

=item addRow

	$table->addRow( @row );

Adds a row to the bottom of the table

=item shiftRow

	@row = $table->shiftRow();

Removes the first row from the table, and returns it.

=item row

	# Get an array of all the table row arrays
	@rows = $table->row();

	# Get the third row of the table
	@row = $table->row(2);

	# Get the last but one row of the table
	@row = $table->row(-2);

	# Count the rows in the table
	$count = $table->row();

	# Count the number of elements in a row
	$count = $table->row(2);

Returns an array of arrays of all the rows in the table if called
without an argument, or returns a row indexed by the integer given if
called with an argument (which can be negative for counting back from
the end of the table.  In scalar context it gives the length of the
table, or number of columns in the row.  When counting the columns,
timestamps are not included in the count.

=item rowIndex

	$lastRowIndex = $table->rowIndex;

	# Don't print out incomplete Homol lines
	foreach $i ($table->rowIndex) {
	    if ($table->cell( $i, 0 ) eq 'Homol') {
		next unless $table->row($i) < 9;
	    }
	    print aceQuote( $table->row($i));
	}

=item cell

	$tag = $table->cell(0,3);
	$table->cell(0,3, 'Finished');	

	$lastElement = $table->cell(-1,-1);

Get or set the contents of a cell in the table.  If the line has
timestamps, it transparently alters the column addressing to
compensate, and also changes the timestamp to the current time and
effective user if the element is altered.  Negative indices count
backwards from the end of the row or column.

=item print

	$table->print;
	$table->print(\*FH);

Prints the ace table to the file handle supplied, or STDOUT if none is
given.  Lines are correctly formatted using B<aceQuote>.

=back

=head1 BUGS

The lines of LongText objects will get un-quoted and quoted.

=head1 AUTHOR

James Gilbert email B<jgrg@sanger.ac.uk>
