
#
# Perl Module for BicMaster
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
package BicMaster;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw( run_bic_prof run_bic_prof_evd );

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

sub run_bic_prof_evd {
    my $profile_file = shift;
    my $db_file = shift;
    my $output = shift;
    my $gap    = shift;
    my $ext    = shift;
    my ($pwd,$string,$length,$out);

    if( !defined $profile_file || !defined $db_file || !defined $output) {
	carp("run_bic_prof must be called with profile_file,db_file, and output");
	return 0;
    }
    if( !defined $gap ) {
	$gap = 1.0;
    }
    if( !defined $ext ) {
	$ext = 1.0;
    }


    $pwd = `pwd`;
    chop $pwd;



    if( !($profile_file =~ /\//)) {
	# profile file is relative to home
	# need to find out where we are...


	$profile_file = "$pwd/$profile_file";
    }

    if( !($db_file =~ /\//)) {
	# file is relative to home
	# need to find out where we are...


	$db_file = "$pwd/$db_file";
    }

    if( !($output =~ /\//)) {
	# file is relative to home
	# need to find out where we are...


	$output = "$pwd/$output";
    }

    $string = "./om -model=sw.model -dev=bic -wait -silent -noalign -out=$output -thr_score=escore -q=$profile_file -db=$db_file -gapop=$gap -gapext=$ext -list=10000";
    $length = length $string;

    print STDERR "About to make system call with [$length] [./om -model=sw.model -dev=bic -wait -noalign -out=$output -thr_score=escore -q=$profile_file -db=$db_file -gapop=$gap -gapext=$ext -list=10000]\n";

    if( ($out = system("om -model=sw.model -dev=bic -wait -noalign -out=$output -thr_score=escore -q=$profile_file -db=$db_file -gapop=$gap -gapext=$ext -list=10000 2>&1 > bicmaster.out.$$")) != 0 ) {
#    if( ($out = system("om -model=sw.model -dev=bic -wait -noalign -out=$output -thr_score=escore -q=$profile_file -db=$db_file -gapop=$gap -gapext=$ext -list=10000")) != 0 ) {
	carp("Could not run om application for OneModel profile search. Bic maybe down. [$!] [$?] [$out]");
	return 0;
    }

    print STDERR "Done it\n";


    return 1; #success.
}

sub run_bic_prof {
    my $profile_file = shift;
    my $db_file = shift;
    my $output = shift;
    my $gap    = shift;
    my $ext    = shift;
    my $pwd;

    if( !defined $profile_file || !defined $db_file || !defined $output) {
	carp("run_bic_prof must be called with profile_file,db_file, and output");
	return 0;
    }
    if( !defined $gap ) {
	$gap = 1.0;
    }
    if( !defined $ext ) {
	$ext = 1.0;
    }


    $pwd = `pwd`;
    chop $pwd;



    if( !($profile_file =~ /\//)) {
	# profile file is relative to home
	# need to find out where we are...


	$profile_file = "$pwd/$profile_file";
    }

    if( !($db_file =~ /\//)) {
	# file is relative to home
	# need to find out where we are...


	$db_file = "$pwd/$db_file";
    }

    if( !($output =~ /\//)) {
	# file is relative to home
	# need to find out where we are...


	$output = "$pwd/$output";
    }
    
#    if( system("om -model=sw.model -dev=bic -wait -silent -noalign -out=$output -nonorm -thr_score=quality -q=$profile_file -db=$db_file -gapop=$gap -gapext=$ext -list=4000'") != 0 ) {
#	carp("Could not run om application for OneModel profile search. Bic maybe down.");
#	return 0;
#    }

    print <<`EOC`;
om -model=sw.model -dev=bic -wait \
-silent -noalign -out=$output -thr_score=escore \
-q=$profile_file -db=$db_file -gapop=$gap \
-gapext=$ext -list=10000
EOC

    return 1; #success.
}




1;  # says use was ok
__END__

=head1 NAME

BicMaster

=head1 DESCRIPTION

Description for B<BicMaster>

=head1 AUTHOR

B<Ewan Birney> Email birney@sanger.ac.uk

=head1 FUNCTIONS

=over

=item run_bic_prof

Runs a profile search on the bic. The communication is entirely by files,
not pipes (sadly). This rsh across to unst to do the search. If the filenames
have no /'s then it appends $pwd to the filenames before calling the rsh.

    run_bic_prof($profile_file,$database_file,$output_to_write_to,$gap,$ext);

=back








