
#
# Perl Module for EMBL::KW
#
# Cared for by Matthew Pocock <mrp@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::KW;

use vars qw($AUTOLOAD @ISA @EXPORT_OK %fields);

use Exporter;
use Carp;
use strict;

use EMBL::_LineGroup;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();


BEGIN {
    @ISA = ( 'Exporter', 'EMBL::_LineGroup' );

    %fields = (
	       #Insert field names here as field => undef,
	       keywordList => undef
    );

  EMBL::KW->check4Clashes();
}

sub initialize {
    my $self = shift;
    $self->{'keywordList'} = [];
}

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = new EMBL::_LineGroup;

    $self->rebless($class);
    return $self;
}

sub fromEMBLStream {
    my $class = shift;
    my $fh = shift;
    my $lineRef = shift;
    my $lineGroup = $class->new;
    my $line;

    $line = $$lineRef;
    while(substr($line, 0, 2) eq 'KW') {
	$line = EMBL::TextFunc->cleanLine($line);
	push @{$lineGroup->keywordList()}, EMBL::TextFunc->listFromText($line, ';\s*', '[\.;]');
	$line = <$fh>;
	if(not defined $line) { warn "KW: End of stream"; last; }
    }
    $$lineRef = $line;
    return $lineGroup;
}

sub toEMBLStream {
    my $self = shift;
    my $fh = shift;
    my $line = shift;

    foreach $line (EMBL::TextFunc->textFromList($self->keywordList(), '; ', '.', 75)) {
	print $fh "KW   $line\n";
    }

    return $fh;
}

1;  # says use was ok
__END__

=head1 NAME

EMBL::KW

=head1 DESCRIPTION

B<EMBL::KW> represents the KW (keyword) lines of an emb entry as defined in
http://www.ebi.ac.uk/ebi_docs/embl_db/usrman/kw_line.html

EMBL::KW inherits from L<EMBL::_LineGroup>

=head1 AUTHOR

B<Matthew Pocock> Email mrp@sanger.ac.uk

=head1 Member variables

=item keywordList

An array of keywords.  They *should* have leading and trailing white spaces stripped.

=back

=head1 Functions

=over

=item fromEMBLStream

This is a module function that accepts a \*filehandle and \$string as arguments.
See EMBL::_LineGroup for a description of what they are for.  It is designed to be
invoked as

    $lg = EMBL::KW->fromEMBLStream($fh, \$line);

within the parser.

=item toEMBLStream

Writes the keywords to a stream in embl formatting.  It takes a \*filehandle as
its only argument.

    $kwLine->toStream(\*STDOUT);
