
#
# Perl Module for EMBL::SV
#
# Cared for by James Gilbert <jgrg@sanger.ac.uk>
#
# Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package EMBL::SV;

use vars qw($AUTOLOAD @ISA @EXPORT_OK %fields);

use Exporter;
use Carp;
use strict;

use EMBL::_LineGroup;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();


BEGIN {
    @ISA = ( 'Exporter', 'EMBL::_LineGroup' );

    %fields = (
	       #Insert field names here as field => undef,
	       'seqVersion' => undef
    );

  EMBL::SV->check4Clashes();
}

sub initialize {};

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    my $self = new EMBL::_LineGroup;

    $self->rebless($class);
    return $self;
}

sub fromEMBLStream {
    my $class = shift;
    my $fh = shift;
    my $lineRef = shift;
    my $lineGroup = new EMBL::SV;
    my $SV;

    chomp( $SV = substr $$lineRef, 5 );
    $lineGroup->seqVersion($SV);

    $$lineRef = <$fh>;

    return $lineGroup;
}

sub toEMBLStream {
    my $self = shift;
    my $fh = shift;

    print $fh "SV   ", $self->seqVersion, "\n";

    return $fh;
}

1;  # says use was ok
__END__

=head1 NAME

EMBL::SV

=head1 DESCRIPTION

B<EMBL::SV> represents the SV (sequence version) line of an embl
entry as specified in http://www.ebi.ac.uk/ebi_docs/embl_db/usrman/sv_line.html
EMBL::SV inherits from B<EMBL::_LineGroup>

=head1 AUTHOR

B<James Gilbert> Email jgrg@sanger.ac.uk

=head1 Member variables

=over seqVersion

The sequence version

=back

=head1 Functions

=over

=item fromEMBLStream

This is a module function that accepts a \*filehandle and \$string as arguments.
See EMBL::_LineGroup for a description of what they are for.  It is designed to be
invoked as

    $lg = EMBL::SV->fromEMBLStream($fh, \$line);

within the parser.

=item toEMBLStream

Writes the database reference to a stream in embl formatting.  It takes a \*filehandle as
its only argument.

    $niLine->toEMBLStream(\*STDOUT);
