#
# BioPerl module for MSPcrunch
#
# Cared for by Michele Clamp  <michele@sanger.ac.uk>
#
# Copyright Michele Clamp
#
# You may distribute this module under the same terms as perl itself
#
# POD documentation - main docs before the code

=pod 

=head1 NAME

Spangle::Analysis::parseMSP

=head1 SYNOPSIS

    my $msp = new Spangle::Analysis::parseMSP($file);      # $file is filename

Extracting the data

    my @features = $msp->each_feature; # Returns an array of the homol features

=head1 DESCRIPTION

Genscan object. Parses the output file from the genscan gene prediction program.

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut

# Let the code begin...

package Spangle::Analysis::parseMSP;

use vars qw(@ISA);
use strict;

# Object preamble - inherits from Bio::Root::RootI;

use Bio::Root::RootI;
use Spangle::Homol;

# Inherits from the base bioperl object
@ISA = qw(Bio::Root::RootI);

sub new {
    my( $pkg, @args ) = @_;

    my $self = bless {}, $pkg;
    
    # Input variables
    # ---------------
    # MSPcrunch output
    
    $self->throw("No MSPcrunch input file")
        unless @args == 1;
    
    $self->{_feature}    = [];
    
    # Now try and get some genes out
    $self->_parse($args[0]);
    
    return $self;
}


sub _parse {
  my ($self,$file) = @_;

  my $seqname;

  open(IN,"<$file") || die "Couldn't open MSPcrunch file $file\n";
  
  while(<IN>) {
    if (! /^\#/) {

      my ($score,$pid,$qstart,$qend,$qid,$hstart,$hend,$id,$title) = split(' ',$_,9);

      my $strand = 1;

      if ($hstart > $hend) {
	$strand = -1;
      }

      my $f = new Spangle::Homol;

      $f->score ($score);
      $f->id    ($id);
      $f->title ($title);
      $f->hstart($hstart);
      $f->hend  ($hend);
      $f->start ($qstart);
      $f->end   ($qend);
  #    $f->pid   ($pid);
      $f->strand($strand);

      $self->_add_Homol_Feature($f);

    }
  } 
  close(IN);
}



=head2 each_Homol_Feature {

  Title   : each_Homol_Feature
  Usage   : @feat = $msp->each_Feature;
 Function: Returns an array of msp homol features
  Returns : @Spangle::Homol
  Args    : none

=cut

sub each_Homol_Feature {
  my ($self) = @_;
  
  return (@{$self->{_feature}});
  
}

=head2 _add_Homol_Feature {

  Title   : _add_Homol_Feature
  Usage   : $msp->add_Homol_Feature($f);
 Function: Adds a feature to the array
  Returns : nothing
  Args    : Spangle::Homol

=cut

sub _add_Homol_Feature {
  my ($self,$f) = @_;
  
  push(@{$self->{_feature}},$f);

}
    
1;
