package Spangle::Html;

use Bio::EnsEMBL::Transcript;

use Spangle::Exon;
use Spangle::GFF;
use Spangle::CGI;

sub transcript2ace {
  my ($tran,$name,$clone,$method) = @_;

  my $out = "";

  # Delete the existing subsequence
  $out .= "\nSequence : $clone\n";
  $out .= "-D Subsequence $name\n";
  $out .= "\n-D Sequence $name\n";

  # print the subsequence netry in the clone object
  $out .= "\nSequence : $clone\n";
  
  my $start;
  my $end;

  my @exon   = $tran->each_Exon;
  my $strand = $exon[0]->strand;

  @exon = sort {$a->start <=> $b->start} @exon;

  if ($strand == 1) {

    $start = $exon[0]     ->start;
    $end   = $exon[$#exon]->end;

  } elsif ($strand == -1) {

    $end   = $exon[0]     ->start;
    $start = $exon[$#exon]->end;

  }

  $out .= "Subsequence $name $start $end\n";

  # now print the subsequence entry

  $out .= "\nSequence : $name\n";
  $out .= "Source $clone\n";

  foreach my $ex (@exon) {

    my $exstart;
    my $exend;

    if ($strand == 1) {
	$exstart = $ex->start - $start + 1;
	$exend   = $ex->end   - $start + 1;
    } elsif ($strand == -1) {
	$exend   = $start - $ex->start + 1;
	$exstart = $start - $ex->end   + 1;
    } else {
	die ("No strand set for exon $ex\n");
    }
    if ($exstart > $exend) {
	my $tmp = $exstart;
	$exstart = $exend;
	$exend   = $tmp;
    }
    $out .= "Source_exons $exstart $exend\n";

  }

  $out .= "CDS\n";
  $out .= "CDS_predicted_by $method\n";
  $out .= "Method $method\n";

  return $out;
}


sub print_summary_html {
  my ($clone,$url,$file,$giffile) = @_;

    my $gff = Spangle::GFF::clone2gff($clone,$url);
    
    Spangle::GFF::make_gif($clone,$gff,$file,$giffile);

    my $td = "<TD>";
    
    # Start a table
    print($file "<h1>Annotated genes (red/dark red)</a></h1>\n");
    print($file "<TABLE BORDER=2 CELLPADDING=5>\n");
    
    print($file "<TR>${td}Gene id</td>
                    ${td}Source  </td>
                    ${td}Start   </td>
                    ${td}End     </td>
                    ${td}Exons</td>
                    </tr>\n");
    
    foreach my $g ($clone->eachProposedGene) {
	if ($g->printFlag == 1) {
	    my @exon = $g->each_Exon;

	    my $tmp = $g->source_tag;
	    $tmp =~ s/[Ee][Mm]://;
	    
	    # Provide a link to the detailed analysis of the gene
	    my $fullurl = $url;
	    $fullurl  =~ s/\.html$//;
	    $fullurl  =~ s/\/.*\///;
	    $fullurl  =  $fullurl . "."  . $tmp . ".html";
	    
	    print($file "<tr>${td}<a href=\"$fullurl\">");
	    
	    # A table row summarizing each gene

	    print($file 
		   $tmp                         . "</a></td>${td}" .    # Id
		   $g->primary_tag              .     "</td>${td}" .    # Source of gene
		   $exon[0]->start              .     "</td>${td}" .    # start coordinate
		   $exon[$#exon]->end           .     "</td>${td}" .    # end coordinate
		   ($#exon+1)                  .     "</td></tr>");     # number of exons
	    
	}
    }
	
    print($file "</table>\n");
    print($file "<p>");    

   # Similar table of pseudogenes
   
    print($file "<h1>Pseudogenes (red/dark red)</a></h1>\n");
    print($file "<TABLE BORDER=2 CELLPADDING=5>\n");
    
    print($file "<TR>${td}Gene id</td>
                    ${td}Source  </td>
                    ${td}Start   </td>
                    ${td}End     </td>
                    ${td}Exons</td>
                    </tr>\n");
      
    foreach my $g ($clone->eachOtherGene) {
	if ($g->printFlag == 1) {
	    my @exon = $g->each_Exon;
	    
	    my $tmp = $g->source_tag;
	    $tmp =~ s/[Ee][Mm]://;

	    # Provide a link to the detailed analysis of the gene
	    my $fullurl = $url;
	    $fullurl  =~ s/\.html$//;
	    $fullurl  =~ s/\/.*\///;
	    $fullurl  =  $fullurl . "."  . $tmp . ".html";
	    
	    print($file "<tr>${td}<a href=\"$fullurl\">");
	    
	    # A table row summarizing each gene
	    print($file 
		   $tmp                         . "</a></td>${td}" .    # Id
		   $g->primary_tag              .     "</td>${td}" .    # Source of gene
		   $exon[0]->start              .     "</td>${td}" .    # start coordinate
		   $exon[$#exon]->end           .     "</td>${td}" .    # end coordinate
		   (scalar @exon)               .     "</td></tr>");    # number of exons
	    
	}
    }
	
    print($file "</table>\n");


    # Print a similar table for the homology spans if they exist
	
  print($file "<h1>New Homology spans</a></h1>\n");
  
  print($file "<TABLE  BORDER=2 CELLPADDING=5>\n");
  print($file "<TR>${td}Span id</td>
                      ${td}Source  </td>
                      ${td}Start   </td>
                      ${td}End     </td>
                      ${td}Exons</td>
                    </tr>\n");
  
  
  foreach my $g ($clone->each_new_span) {
    if ($g->printFlag == 1) {
		my @exon = $g->each_Exon;
		
		my $tmp = $g->source_tag;
		$tmp =~ s/[Ee][Mm]://;

		my $fullurl = $url;
		$fullurl  =~ s/\.html$//;
		$fullurl  =~ s/\/.*\///;
		$fullurl  =  $fullurl . "."  . $tmp . ".html";
		
		print($file  "<tr>" .$td . "<A href=$fullurl>" . $tmp      . "</a></td>${td}" . 
		       $g->primary_tag     . "</td>${td}" . 
		       $exon[0]->start     . "</td>${td}" . 
		       $exon[$#exon]->end  . "</td>${td}" . 
		       ($#exon+1)          . "</td>");
		
		print($file "</tr>\n");
	    }
	# and for predicted genes    
	  }
  print($file "</TABLE>\n");
  print($file "<p>\n");
    print($file "<h1>Predicted genes (blue/dark blue)</a></h1>\n");
    print($file "<TABLE BORDER=2 CELLPADDING=5>\n");
    
    print($file "<TR>${td}Gene id</td>
                    ${td}Source  </td>
                    ${td}Start   </td>
                    ${td}End     </td>
                    ${td}Exons</td>
                    </tr>\n");
      
    foreach my $g ($clone->eachPredictedGene) {
	if ($g->printFlag == 1) {
	    my @exon = $g->each_Exon;
	    
	    my $tmp = $g->source_tag;
	    $tmp =~ s/[Ee][Mm]://;

	    # Provide a link to the detailed analysis of the gene
	    my $fullurl = $url;
	    $fullurl  =~ s/\.html$//;
	    $fullurl  =~ s/\/.*\///;
	    $fullurl  =  $fullurl . "."  . $tmp . ".html";
	    
	    print($file "<tr>${td}<a href=\"$fullurl\">");
	    
	    # A table row summarizing each gene
	    print($file 
		  $tmp                         . "</a></td>${td}" .    # Id
		  $g->primary_tag              .     "</td>${td}" .    # Source of gene
		  $exon[0]->start              .     "</td>${td}" .    # start coordinate
		  $exon[$#exon]->end           .     "</td>${td}" .    # end coordinate
		  (scalar @exon)               .     "</td></tr>");    # number of exons
	    
	}
    }
	
    print($file "</table>\n");
    print($file "<p>");   
  }


sub print_gene_html {
  my ($clone,$span,$path,$display,$file) = @_;
#  print("HTML::display id $display\n");
  print_exons_html2($clone,$span,$path,$display,$file);
}
    
 sub print_exons_html {
  my ($clone,$gene,$path,$display,$file) = @_;

  # The name of the gene 
  my $genename =  $gene->source_tag;

  # Header stuff
  print($file "<p><h1>$genename");
  print($file " [<a href=\"http://webace.sanger.ac.uk:80/cgi-bin/display?db=humace&class=Sequence&object=" .
	 $genename . "&display=active&.submit=show+object\" target=\"_webace\">webace</a>]</h1>\n");

  # CGI variables
  print($file "<center>");
  print($file "<FORM METHOD=\"POST\" enctype='multipart/form-data' ACTION=\"" . Spangle::CGI::cgiscript() . "\">\n");
  print($file "<INPUT type=submit  name=action     value=\"Apply selected hits as supporting evidence\">\n");
  print($file "<INPUT type=submit  name=action     value=\"Edit/create gene\"><p>\n");
  print($file "<INPUT type=hidden name=\"subseq$genename\">\n");
  print($file "<input type=hidden name=\"gene\"    value=\"" .  $genename . "\">\n");
  print($file "<input type=hidden name=\"acepath\" value=\"" .  $path . "\">\n");
  print($file "<input type=hidden name=\"display\" value=\"" .  $display . "\">\n");
  print($file "<input type=hidden name=\"noform\"  value=\"true\">\n");
  print($file "</center>");

  my $td = "<TD BGCOLOR=pink>";
  
  my %idhash;
  my %idcount;
  my %lenhash;

  my %source;
  my %primary;
  my %type;

  # This loop is to find the order we need
  # to display the database hits in

  foreach my $ex ($gene->each_Exon) {
    foreach my $f ($ex->each_supporting_feature) {
      my $id = $f->id;

      $idcount{$id}++;
      $idhash {$id}{$ex} = $f;

      my $source  = $f->source_tag;
      my $primary = $f->primary_tag;
      
      if ($source =~ /BLASTX/) {
	$type{$id} = "Protein_match";
      } elsif ($source =~ /mRNA/) {
	$type{$id} = "cDNA_match";
      } elsif ($source =~ /EST/) {
	$type{$id} = "EST_match";
      }

      $primary{$id} = $primary;
      $source {$id} = $source;
      $lenhash{$id} = $f->length;

    }
  }

  print($file "<table border=0 cellpadding=0>\n");
  print($file "<tr><td>&nbsp;</td>\n");
  
  # print the top row of exons with start and end coords
  my $count  = 0;
  my $count2 = 0;
  
  my @exon = $gene->each_Exon;
  
  if ($exon[0]->strand == 1) {
    @exon = sort {$a->start <=> $b->start} @exon;
  } else {
    @exon = sort {$b->start <=> $a->start} @exon;
  }
  
  print($file "<td><TD>");
  
  foreach my $ex (@exon) {
    print($file "<td><font size=2>", $ex->start, "<br>", $ex->end, "</font>");

    # print the splicing information here
    if ($count < $#exon) {
      my $str = $exon[$count]->donor . "-" . $exon[$count+1]->acceptor;
      $str =~ tr/A-Z/a-z/;
      
      my $col = "white";
      
      if ($str ne "gt-ag" && $str ne "ct-ac") {
	$col = "green";
      }
      
      print($file "<td bgcolor=$col>$str</td>");
    }
    $count++;
  }
  
  print($file "</tr>\n");

  # Now do the rest of the table column by column
  
  foreach my $id (sort {$idcount{$b} <=> $idcount{$a}} keys %idcount) {
      my $col;
      if ($source{$id} eq "BLASTX") {
	  $col = "blue";
      } elsif ($source{$id} eq "vertebrate_mRNA_eg") {
	  $col = "red";
      } else {
	$col = "white";
      }


      # Print the database id (left hand most table entry) with the link to seq-query
      print($file "<tr></font>
                      <td bgcolor=$col> <font size=2>  <input type=checkbox name=\"sequence_$id\"                         </font></td>
                      <td>              <font size=2>  &nbsp &nbsp<a href=\"http://www.sanger.ac.uk/cgi-bin/seq-query?$id\">$id</a>  </font></td>
                      <td>              <font size=2> $lenhash{$id}) $source{$id}");

      print($file "<input type=hidden name=\"type_$id\" value=\"$type{$id}\"></td>\n");
      
      $count = 0;

      foreach my $ex (@exon) {

	if ($idhash{$id}{$ex}) {

	  my $f = $idhash{$id}{$ex};
	  
	  print_cell($f,$ex,0,$file);

	} else {
	  print($file "<td bgcolor=white>&nbsp;</td>");
	}
	
	# The spacing cell with splice info in
	if ($count < $#exon) {
	  
	  my $f1;
	  my $f2;

	  foreach my $ff ($ex->each_supporting_feature) {
	    if ($ff->id eq $id) {
	      $f1 = $ff;
	    }
	  }

	  foreach my $ff ($exon[$count+1]->each_supporting_feature) {
	    if ($ff->id eq $id) {
	      $f2 = $ff;
	    }
	  }

	  if ($f1 && $f2) {
	    my $diff;
	    
	    if ($f1->strand == 1 && $ex->strand == 1) {
	      $diff = $f2->hstart - $f1->hend;

	      if ($diff == 1) {
		print($file "<td bgcolor=white>-></td>");
	      } else {
		print($file "<td bgcolor=white>$diff</td>");
	      }
	      
	    } elsif ($f1->strand == -1 && $ex->strand == -1) {
	      $diff = $f2->hstart - $f1->hend;

	      if ($diff == 1) {
		print($file "<td bgcolor=white>-></td>");
	      } else {
		print($file "<td bgcolor=white>$diff</td>");
	      }

	    } elsif ($f1->strand == 1 && $ex->strand == -1) {
	      $diff = $f1->hstart - $f2->hend;

	      if ($diff == 1) {
		print($file "<td bgcolor=white><-</td>");
	      } else {
		print($file "<td bgcolor=white>$diff</td>");
	      }

	    } else {
	      $diff = $f1->hstart - $f2->hend;

	      if ($diff == 1 ) {
		print($file "<td bgcolor=white><-</td>");
	      } else {
		print($file "<td bgcolor=white>$diff</td>");
	      }
	    }

	  } else {
	    print($file "<td bgcolor=white>&nbsp;</td>");
	  }
	}
	$count++;
	
      }
      
      print($file "</tr>\n");
    }
  
  print($file "</table></form>\n");
}
sub print_exons_html2 {
  my ($clone,$gene,$path,$display,$file) = @_;



  # The name of the gene 
  my $genename =  $gene->source_tag;

  # Header stuff
  print($file "<p><h1>$genename");
  print($file " [<a href=\"http://webace.sanger.ac.uk:80/cgi-bin/display?db=humace&class=Sequence&object=",
	 $genename, "&display=active&.submit=show+object\" target=\"_webace\">webace</a>]</h1>\n");

  #JavaScript for select all boxes
  print($file "<script language=\"JavaScript\">\n",
    "function selectAll(form){\n",
    "   form = form.elements\n",
    "   var n, tmp\n",
    "   for (var i=0; i<form.elements.length;i++) {\n",
    "     tmp = form.elements[i]\n",
    "     if (tmp.type == 'checkbox') {\n",
    "             tmp.checked=true\n",
    "     }\n",
    "   }\n",
    "} \n",
    "</script>\n");

  # CGI variables
  print($file "<center>",
    "<FORM METHOD=\"POST\" enctype='multipart/form-data' ACTION=\"", Spangle::CGI::cgiscript(), "\">\n",
    "<INPUT type=submit  name=action     value=\"Apply selected hits as supporting evidence\">\n",
    "<INPUT type=submit  name=action     value=\"Edit/create gene\"><p>\n",
    "<INPUT type=hidden name=\"subseq$genename\">\n",
    "<input type=hidden name=\"gene\"    value=\"$genename\">\n",
    "<input type=hidden name=\"acepath\" value=\"$path\">\n",
    "<input type=hidden name=\"display\" value=\"$display\">\n",
    "<input type=hidden name=\"noform\"  value=\"true\">\n",
    "<input type=hidden name=\"clone\" value=\"$clone\">\n",
     "</center>");


  my $td = "<TD BGCOLOR=pink>";
  
  my %idhash;
  my %idcount;
  my %lenhash;

  my %source;
  my %primary;
  my %type;

  my $short_type;

  # This loop is to find the order we need
  # to display the database hits in

  foreach my $ex ($gene->each_Exon) {
    foreach my $f ($ex->each_supporting_feature) {
      my $id = $f->id;

         $idcount{$id}++;
         $idhash {$id}{$ex} = $f;

      my $source  = $f->source_tag;
      my $primary = $f->primary_tag;
      
      if ($source =~ /BLASTX/) {
	$type{$id} = "Protein_match";
      } elsif ($source =~ /mRNA/) {
	$type{$id} = "cDNA_match";
      } elsif ($source =~ /EST/) {
	$type{$id} = "EST_match";
      }

      $primary{$id} = $primary;
      $source {$id} = $source;
      $lenhash{$id} = $f->length;

    }
  }

  print($file "<table border=0 cellpadding=0>\n");
 # print($file "<tr><td>&nbsp;</td>\n");
  
  # print the top row of exons with start and end coords
  my $count  = 0;
  my $count2 = 0;
  
  my @exon = $gene->each_Exon;
  
  if ($exon[0]->strand == 1) {
    @exon = sort {$a->start <=> $b->start} @exon;
  } else {
    @exon = sort {$b->start <=> $a->start} @exon;
  }
 
  print($file "<tr align=center>");
  for (my $count = 1; $count <= ($#exon+1); ++$count){  
      print($file "<td>ex$count</td><td></td>");
  }
  print($file "</tr>\n<tr>");
  
  foreach my $ex (@exon) {
    print($file "<td><font size=2>", $ex->start, "<br>", $ex->end, "</font></td>");

    # print the splicing information here
    if ($count < $#exon) {
      my $str = $exon[$count]->donor . "-" . $exon[$count+1]->acceptor;
      $str =~ tr/A-Z/a-z/;
      
      my $col = "white";
      
      if ($str ne "gt-ag" && $str ne "ct-ac") {
	$col = "green";
      }
      
      print($file "<td bgcolor=$col>$str</td>");
    }
    $count++;
  }
  
  print($file "</tr></table>\n");
  
#  print($file "<td><TD>");
#  print($file "<PRE>\n", (" " x 47));

#  foreach my $ex (@exon) {
#      printf($file "%7d",$ex->start);
      # print the splicing information here
#      if ($count < $#exon) {
#	  my $str = $exon[$count]->donor . "-" . $exon[$count+1]->acceptor;
#	     $str =~ tr/A-Z/a-z/;
#	  
#	  my $col = "black";
#	  
#	  if ($str ne "gt-ag" && $str ne "ct-ac") {
#	      $col = "green";
#	  }
#      
#	 # print($file "<td bgcolor=$col>$str</td>");
#	  printf($file "<font color=$col>%6s</font>",$str);
#      }
#      $count++;
#  }
#  print($file "\n");
#  $count = 0;
#  print($file " " x 47);
#  foreach my $ex (@exon) {
#      printf($file "%7d", $ex->end);
#      if ($count < $#exon) {
#	  printf($file "%6s", " ");
#      }
#      $count++;
#  }
#  print($file "\n");


  print($file "</pre><INPUT type=button value=\"Check all\"  onClick=\"javascript:selectAll(document.forms[1])\">");
  print($file "&nbsp<INPUT TYPE=reset value=\"Uncheck all\">");   

  # Now do the rest of the table column by column

  # print the exon number first
  print($file "<BR><BR><PRE>\n"." " x 36); 
    
  my $count = 1;

  foreach my $ex (@exon) {
      printf($file "%4d  ", $count);
      $count++;
  }
  
  print($file "\n<br>\n");
  
  foreach my $id (sort {$idcount{$b} <=> $idcount{$a}} keys %idcount) {
      my $col;
      if ($source{$id} eq "BLASTX") {
	  $col = "blue";
          $short_type = "protein";
      } elsif ($source{$id} eq "vertebrate_mRNA_eg") {
	  $col = "red";
          $short_type = "mRNA";
      } else {
	  $col = "black";
          $short_type = "EST";
      }


      # Print the database id (left hand most table entry) with the link to seq-query
      printf($file "<input type=checkbox name=\"sequence_%s\">&nbsp &nbsp <a href=\"http://www.sanger.ac.uk/cgi-bin/seq-query?%s\">%12s</a> (%6d) <font color=%s>%9s</font>&nbsp&nbsp",$id,$id,$id,$lenhash{$id},$col,$short_type);

      print($file "<input type=hidden name=\"type_$id\" value=\"$type{$id}\">");
      
      $count = 0;

      foreach my $ex (@exon) {

	if ($idhash{$id}{$ex}) {

	  my $f = $idhash{$id}{$ex};
	  
	  print_cell2($f,$ex,0,$file);

	} else {
	    printf($file "<font color=black>%2s</font>","..");
	}
	
	# The spacing cell with splice info in
	if ($count < $#exon) {
	  
	  my $f1;
	  my $f2;

	  foreach my $ff ($ex->each_supporting_feature) {
	    if ($ff->id eq $id) {
	      $f1 = $ff;
	    }
	  }

	  foreach my $ff ($exon[$count+1]->each_supporting_feature) {
	    if ($ff->id eq $id) {
	      $f2 = $ff;
	    }
	  }

	  if ($f1 && $f2) {
	    my $diff;
	    
	    if ($f1->strand == 1 && $ex->strand == 1) {
		$diff = $f2->hstart - $f1->hend;

		if ($diff == 1) {
		    printf($file "%4s","->");
		} else {
		    printf($file "%4s",$diff);
		}
		
	    } elsif ($f1->strand == -1 && $ex->strand == -1) {
		$diff = $f2->hstart - $f1->hend;
		
		if ($diff == 1) {
		    printf($file "%4s","->");
		} else {
		    printf($file "%4s",$diff);
		}
		
	    } elsif ($f1->strand == 1 && $ex->strand == -1) {
		$diff = $f1->hstart - $f2->hend;
		
		if ($diff == 1) {
		    printf($file "%4s","<-");
		} else {
		    printf($file "%4s",$diff);
		}
		
	    } else {
	      $diff = $f1->hstart - $f2->hend;

	      if ($diff == 1 ) {
		  printf($file "%4s","<-");
	      } else {
		  printf($file "%4s",$diff);
	      }
	    }

	  } else {
	      printf($file "%4s"," ");
	  }
	}
	$count++;
	
      }
      
      print($file "\n");
    }
  

  print($file "</pre><INPUT type=button value=\"Check all\"  onClick=\"javascript:selectAll(document.forms[1])\">");
  print($file "&nbsp<INPUT TYPE=reset value=\"Uncheck all\">");   
  print($file "\n</form>\n");
}

sub print_cell2 {
  my ($feat,$exon,$gif,$file) = @_;

  my $hstart;
  my $hend;
  
  my $loverlap;
  my $roverlap;
  
  my $strand;
  
  my @overlap = $feat->match($exon);

  if ($overlap[0]) {

    if (abs($overlap[1]) <= 5 && abs($overlap[2]) <= 5) {
      $colour = "red";
    } elsif ($overlap[1] <= 0 && $overlap[2] <= 15) {
      $colour = "orange";
    } else {
      $colour = "yellow";
    }
  }  
  if ($exon->strand == 1) {
    
    if ($feat->strand == 1) {
      $strand   = "+/+";
      $hstart   = $feat->hstart;
      $hend     = $feat->hend;
      $loverlap = $overlap[1];
      $roverlap = $overlap[2];
      
    } else {
      $strand = "+/-";
      $hstart = $feat->hend;
      $hend   = $feat->hstart;
      $loverlap = $overlap[1];
      $roverlap = $overlap[2];
      
    }
  } else {
    if ($feat->strand == 1) {
      $strand = "-/+";
      $hstart = $feat->hend;
      $hend   = $feat->hstart;
      $loverlap = $overlap[2];
      $roverlap = $overlap[1];
      
    } else {
      $strand = "-/-";
      $hstart = $feat->hstart;
      $hend   = $feat->hend;
      $loverlap = $overlap[1];
      $roverlap = $overlap[2];
    }
    
  }
  my $content;
  my $td;
  
  if ($gif) {
      $content = "<img border=0 src=\"http://intweb.sanger.ac.uk/LocalUsers/humpub/annotation/evidence/$colour.gif\">";
  } else {
      $content = "#" x 2;
      $content = "<font color=$colour><b>$content</b></font>";
  }    
  
  print($file "<a href=\"#\" onMouseOver=print_exon(\"$strand\"," . 
	$hstart                        . "," .
	$hend                          . "," .
	$loverlap   . "," .
	$roverlap   . "," .
	int($feat->score*100/(1+abs($feat->hstart-$feat->hend))) .
	")  >$content</a>");
  
}

sub print_cell {
  my ($feat,$exon,$gif,$file) = @_;

  my $hstart;
  my $hend;
  
  my $loverlap;
  my $roverlap;
  
  my $strand;
  
  my @overlap = $feat->match($exon);

  if ($overlap[0]) {

    if (abs($overlap[1]) <= 5 && abs($overlap[2]) <= 5) {
      $colour = "red";
    } elsif ($overlap[1] <= 0 && $overlap[2] <= 15) {
      $colour = "orange";
    } else {
      $colour = "yellow";
    }
  }  
  if ($exon->strand == 1) {
    
    if ($feat->strand == 1) {
      $strand   = "+/+";
      $hstart   = $feat->hstart;
      $hend     = $feat->hend;
      $loverlap = $overlap[1];
      $roverlap = $overlap[2];
      
    } else {
      $strand = "+/-";
      $hstart = $feat->hend;
      $hend   = $feat->hstart;
      $loverlap = $overlap[1];
      $roverlap = $overlap[2];
      
    }
  } else {
    if ($feat->strand == 1) {
      $strand = "-/+";
      $hstart = $feat->hend;
      $hend   = $feat->hstart;
      $loverlap = $overlap[2];
      $roverlap = $overlap[1];
      
    } else {
      $strand = "-/-";
      $hstart = $feat->hstart;
      $hend   = $feat->hend;
      $loverlap = $overlap[1];
      $roverlap = $overlap[2];
    }
    
  }
  my $content;
  my $td;
  
  if ($gif) {
    $content = "<img border=0 src=\"http://intweb.sanger.ac.uk/LocalUsers/humpub/annotation/evidence/$colour.gif\">";
    $td = "<TD>";
  } else {
    $content = "&nbsp";
    $td = "<TD BGCOLOR=$colour>";
  }    
  
  print($file "$td<a href=\"#\" onMouseOver=print_exon(\"$strand\"," . 
	$hstart                        . "," .
	$hend                          . "," .
	$loverlap   . "," .
	$roverlap   . "," .
	int($feat->score*100/(1+abs($feat->hstart-$feat->hend))) .
	")  >$content</a>");
  
}

sub exon_status {

  my $str = "</FORM><SCRIPT LANGUAGE=\"JavaScript\"> \n" .
  "function print_exon(strand,start,end,splice1,splice2,score) {\n" . 
  "stt = \"Strand: \" + print_exon.arguments[0] + \n" .
  "      \" \"  + print_exon.arguments[1]       + \n" .
  "      \"-\"  + print_exon.arguments[2]       + \n" .
  "      \" (\" + print_exon.arguments[3]       + \n" . 
  "      \",\"  + print_exon.arguments[4]       + \n" .
  "      \") \" + print_exon.arguments[5];        \n" .
  " status = stt; } ".
  "</SCRIPT>";

  return $str;
}

1;
