package Spangle::Trembl_DB2;

use DBI;

use strict;
#use Hum::Tracking qw( ref_from_query );


use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

$VERSION = 1.00;


sub new {

  my($class) = @_;
  my $self = {};

  bless $self,$class;
  
  $self->_connect;             # Connect to the mysql database

  return $self;
}

sub disconnect {
  my ($self) = @_;

  $self->{_dbh}->disconnect;
}
sub _connect {
  my ($self) = @_;
  
  # Connect to database
  my $dbh = DBI->connect('DBI:mysql:database=spangle:host=humsrv1;port=3399','humpub','');
  
  if ($dbh) {
    print("Connection successful. Hoorah!!\n");
  }

  $self->{_dbh} = $dbh;
}


sub get_accession {
  my ($self,$cloneid) = @_;

  my $outref = $self->do_query("select acc from clone where id = \'$cloneid\'");
  my $acc;

  foreach my $rowhash (@$outref) {
    $acc     = $rowhash->{acc};
  }

  return $acc;
}


sub set_accession {
  my ($self,$cloneid,$accession) = @_;

  $self->do_query("replace into clone(id,acc) values('$cloneid','$accession')");

}

	
sub each_tremblid {
  my ($self,$cloneid) = @_;

  my $outref = $self->do_query("select trembl_id from trembl_entry where clone_id = \'$cloneid\'");

  my @trembl;

  foreach my $rowhash (@$outref) {
    my $hit     = $rowhash->{trembl_id};
    push(@trembl,$hit);
  }
  
  return @trembl;
}


sub add_tremblid {
  my ($self,$cloneid,$tremblid) = @_;

  $self->do_query("replace into trembl_entry(clone_id,trembl_id) values('$cloneid','$tremblid')");
  
}
sub add_swissprotid {
  my ($self,$cloneid,$swissprotid) = @_;

  $self->do_query("replace into swissprot_entry(clone_id,swissprot_id) values('$cloneid','$swissprotid')");

}

sub generate_DB {
  my ($self) = @_;

  my %clones;

  my $species = ucfirst lc "human";
    
  my $query = qq(
		 select c.clonename, f.suffix, f.accession
		 from   project             p, 
                        finished_submission f,
		        clone_project       cp, 
		        clone               c
		 where
            c.clonename    = cp.clonename and
            cp.projectname = p.projectname and
            p.projectname  = f.projectname and
            c.speciesname  = '$species' and

            p.projectname not in(
                select projectname
                from project_status
                where status = 24 )
		);
    
  #my $ans = ref_from_query( $query );
  my $ans;
  foreach my $row (@$ans) {
    my( $clone, $suffix, $accession ) = @$row;
    my $seq_name = $suffix ? $clone.$suffix : $clone;
    $clones{$seq_name} = $accession;
  }

  foreach my $clone (keys %clones) {
    open (SEQ,"efetch EM:" . $clones{$clone} . " |");

    my $found = 0;
    my $accession;
    my $refline;

    my @trembls;
    my @swissprots;

    my @others;
    
    while (defined((my $line = <SEQ>)) && ($found == 0)) {
      
      if ($line =~ /^SQ /) {
	$found = 1;
	close(SEQ);
      } elsif ($line =~ /^AC +(\S+)/) {
	$accession = $1;
	$accession =~ s/;//g;
      } elsif ($line =~ /^RL +(.*)\n$/) {
	$refline .= $1;
      } elsif ($line =~ /^DR +SPTREMBL\; +(\S+)\; +/) {
	my $tmp = $1;
	$tmp =~ s/\;//g;
	push(@trembls,split(' ',$tmp));
      } elsif ($line =~ /^DR +SWISS-PROT\; +(\S+)\; +/) {
	my $tmp = $1;
	$tmp =~ s/\;//g;
	push(@swissprots,split(' ',$tmp));
      } elsif ($line =~ /^DR +(.*)\n$/) {
	push(@others,$1);
      }
    }
    
    if ($found == 1) {
      $self->set_accession($clone,$accession);

      my $newacc = $self->get_accession($clone);
      print("Clone is $clone : $accession TR: @trembls SW: @swissprots\n");
      if ($#others > 0) {
	print("WARNING: Unknown database refs: @others\n");
      }
      foreach my $tr (@trembls) {
	$self->add_tremblid($clone,$tr);
      }
      foreach my $tr (@swissprots) {
	$self->add_swissprotid($clone,$tr);
      }
    }
  
  }


  sub each_trembl {
    my ($self) = @_;

    my @ids;

    my $outref = $self->do_query("select trembl_id from trembl_entry");

    foreach my $rowhash (@$outref) {
      push(@ids,$rowhash->{trembl_id});
    }

    return @ids;
  }

   sub each_swissprot {
    my ($self) = @_;

    my @ids;

    my $outref = $self->do_query("select swissprot_id from swissprot_entry");

    foreach my $rowhash (@$outref) {
      push(@ids,$rowhash->{swissprot_id});
    }

    return @ids;
  }

   sub each_accession {
    my ($self) = @_;

    my @ids;

    my $outref = $self->do_query("select acc from clone where acc != ''");

    foreach my $rowhash (@$outref) {
      push(@ids,$rowhash->{acc});
    }

    return @ids;
  }

  sub do_query {
    my( $self,$query ) = @_;
    my( $sth, @answer );
    
    $sth = $self->{_dbh}->prepare( $query );
    $sth->execute();

    my @rows;
    while (my $rowhash = $sth->fetchrow_hashref) {
      push(@rows,$rowhash);
    }

    return \@rows;

  }
  
  END {
    $self->{_dbh}->disconnect() if $self->{_dbh};
  }   
}


1;
