#!/usr/local/bin/perl 

use lib '/nfs/disk100/humpub/modules/PerlModules';

use strict;

use Hum::Env;

use Carp;
use CGI;
use Getopt::Long;

use CGI::Carp qw(fatalsToBrowser); 

use Ace;

use Spangle::Jobs;
use Spangle::Clone;
use Spangle::Html;
use Spangle::CGI;
use Spangle::Ace;
use Bio::EnsEMBL::Exon;

$| = 1;

my ($prog) = $0 =~ m|([^/]+)$|;

my $query = new CGI; print $query->header;     # New CGI query object

my $host    = $query->param('host');
my $port    = $query->param('port');
my $path    = $query->param('acepath');
my $clone   = $query->param('clone') || $query->param('cloneid');
my $display = $query->param('display');


printTopButton();            # Header type stuff

if ($path){ 
    print("<h4>Ace path is $path</h4>\n");
}

#print("<h4>Display is $display</h4>\n");

#print "About to edit the gene\n";
editGene();                  # Apply gene edits to a new gene form
#print "About to process\n";
processClone();              # Fetches supporting evidence and writes html
#print "About to fetch sub seqs\n";
fetchSubsequences();         # Fetches subsequence list and writes html
#print "About to write ace\n";
writeAceFile();              # Processes the form generated by processClone
                             # and writes an ace file of the selectedsupporting evidence
#print "going to modify\n";

modifyGene(\*STDOUT);

#print "exiting\n";

################################################################################
#
# Subroutines
#
################################################################################

sub printTopButton {
  my $action = $query->param('action');

  my $file  = \*STDOUT;
  if ($action eq "Top" || $action eq "") {
    my $path  = $query->param('acepath');

    Spangle::CGI::printForm($path,$display,$file,"");
    exit(1);
  } else {
    Spangle::CGI::printTopButton($file,$path,$display);

  }
}


sub editGene {
  
  if ($query->param('action') eq "Make a virgin gene") {makeVirginGene();}
  if ($query->param('action') eq "Fuse genes") {fuseGenes();}
  return unless ($query->param('action') eq "Edit/create gene");

  my $subseq;

  foreach my $n ($query->param) {
    if ($n =~ /subseq(.+)/) {
      $subseq = $1;
    }
  }
  
  if ($subseq eq "") { die "No subsequence selected";}

  my $cloneid = $query->param('clone') || $query->param('cloneid')
    or die "No param 'clone'";
  
  print ("Fetching subsequence $subseq for $cloneid<br>");
  print ("Connecting to database....");
  my $db = Spangle::Ace::connect($host,$port,$query->param('acepath'));
  print ("done<p>\n");

  my $clone = Spangle::Clone->new;
  
  $clone->id   ($cloneid);
  $clone->acedb($db);
  $clone->fetch_subsequences;

  my $path = $query->param('acepath');
  foreach my $s ($clone->eachSubsequence) {
    if ($s->source_tag eq $subseq) {
      Spangle::CGI::print_gene_form($clone,$s,$path,$display,\*STDOUT);
    }
  }

  exit(1);
}

sub fuseGenes {
  
  my @subseq;
  foreach my $n ($query->param) {
        if ($n =~ /subseq(.+)/) {
            push(@subseq,$1);
        }   
  }

  if ($subseq[0] eq "") { die "No subsequence selected";}

  my $cloneid = $subseq[0] ;
     $cloneid =~ s/(.*?)\..*/$1/;
  
  print ("Fetching subsequences @subseq for $cloneid<br>");
  print ("Connecting to database....");
  my $db = Spangle::Ace::connect($host,$port,$query->param('acepath'));
  print ("done<p>\n");

  my $clone = Spangle::Clone->new;
  
  $clone->id   ($cloneid);
  $clone->acedb($db);
  $clone->fetch_subsequences;

  my $path = $query->param('acepath');
  my @subsequences;
 # my $i = 0;
  foreach my $s ($clone->eachSubsequence) {
        foreach my $subs(@subseq){
            if ($s->source_tag eq $subs) {
                push(@subsequences,$s);
            }       
#            $i++;
        }
  }
  Spangle::CGI::print_fusion_gene_form($clone,\@subsequences,$path,$display,\*STDOUT);

  exit(1);
}

sub makeVirginGene {

    my $exon_numb = $query->param('virgin_exon_number');
    my $subseq = new Spangle::Subseq();
    $subseq->strand(1);

    for (my $i=1; $i<=$exon_numb; $i++){
        my $ex = new Bio::EnsEMBL::Exon($i,$i+1,1);
        $subseq->add_Exon($ex);
    }
       
    my $clone   = $query->param('clone') || $query->param('cloneid');  
    $subseq->source_tag($clone);
   
  
    Spangle::CGI::print_gene_form($clone,$subseq,$path,$display,\*STDOUT);
    return;
}

sub writeAceFile {
    return unless ($query->param('gene') ne "");
    
    my $gene = $query->param('gene');
    my $path = $query->param('acepath');


    my @evidence;               # list of supporting database entries
    my %type;                   # the type of evidence

    # The selected supporting evidence is in tags
    #     sequence_P08942    (where P08942 is the hit id)
    #
    # The type of evidence is in tags
    #     type_P08942 

    foreach my $p ($query->param) {
	if ($p =~ s/^sequence_(.*)/$1/) {
	    push(@evidence,$1);
	}
	
	if ($p =~ s/^type_(.*)/$1/) {
	    $type{$1} = $query->param("type_$1");
	}
    }
    
  Spangle::CGI::writeAceFile($gene,$path,\@evidence,\%type,$display);
    

}

sub fetchSubsequences {

    return if ($query->param('action') eq "Make a virgin gene");
    return unless ($query->param('choose') eq "fetchsubseq");

    #my $clone = $query->param('clone');

    print ("Fetching subsequences for $clone<br>\n");
    print ("Connecting to database...");
    my $db = Spangle::Ace::connect($host,$port,$query->param('acepath'));
    print ("done<p>\n");

    my $cloneobj  = new Spangle::Clone;
  
    $cloneobj->id   ($clone);
    $cloneobj->acedb($db);
    $cloneobj->fetch_subsequences;
  
    my $path;   

    if ($query->param('acepath') ne "") {
        $path = $query->param('acepath');
    }	

    my $out = Spangle::CGI::print_subsequence_form($cloneobj,$path,$display);
  
    print($out);
}

sub processClone {
  my $choose   = $query->param('choose');
  my $subseq   = $query->param('subseq');
  my $evidence = $query->param('evidence');
  my $action   = $query->param('action');
  my $genename = $query->param('genename');

  my $do_subseq = 0;

  return if ($action eq "Make a virgin gene");
  return unless ($choose eq "clone"  || 
		 $choose eq "subseq" || 
		 $choose eq "all"    || 
		 $subseq ne ""       || 
		 $action eq "Find supporting evidence");
  
  if ($action eq "Find supporting evidence") {$evidence = 1;}
  
  my $opt;
  my $pidfile  = "/tmp/$$";    
  
  
  # Check we have some clones
  if ($clone eq  "" && $subseq eq "" && $genename eq "") {
      print("ERROR: No clones/subsequences entered<p>\n");
      exit 1;
  }
  
  
  # Add extra options if we only want to process a subsequence
  if ($choose   eq "all") {
      $opt      .= " -all ";
      $do_subseq = 0;
      $subseq = "";

  } elsif ($choose eq "subseq") {
      my $clone = $query->param('clone') || $query->param('cloneid');
      $opt      .= " -subseq \"$clone\" ";
      $clone     =~ s/^(.*?)\..*/$1/;
      $do_subseq = 1;

  } elsif ($subseq ne "") {
      my $tmp;

      $clone = $subseq;
      
      foreach my $n ($query->param) {
	  if ($n =~ /subseq(.+)/) {
	      $tmp .= $1 . ",";
	  }
      }
    
      $tmp =~ s/\,$//;
      $tmp = "\"" . $tmp . "\"";
    
      $opt      .= " -subseq $tmp ";
      $subseq    = $tmp;
      $do_subseq = 0;
    
  } elsif ($genename ne "") {

    $clone     = $genename;
    $clone     =~ s/(.*?)\..*/$1/;
    $opt      .= " -subseq $genename";
    $subseq    = $genename;
    $do_subseq = 1;

  }
  
  # Set the acedb host and port variables
  $host = 'humsrv1' unless $host;
  $port = 310000    unless $port;
  
  my $dbopt   = " -host $host -port $port ";
  
  my $htfile;
  my $htserver;
  my $fsfile;
  
  # Set the paths for the html output
  
  if ($path ne "") {
      if ($path !~ /\/$/) {
	  $path .= "/";
	  
      }

      $dbopt = " -path $path ";
      
      $htserver = "file:";
      $htfile   = "$path";
      $fsfile   = "$path";
      
  } else {
      
      $htserver = "http://intweb.sanger.ac.uk/";
      $htfile   = "~humpub/annotation/evidence";
      $fsfile   = "/nfs/intweb/server/htdocs/LocalUsers/humpub/annotation/evidence/";
      
  }
  
  if ($display) {
      $dbopt .= " -display $display ";
  }
  
  # Create html real file names and urls
  my ($file,$htmlfile) = Spangle::CGI::get_random_httpfile("$fsfile$clone.",".html","$htserver$htfile$clone.");
  
  # Extra options depending on what we are doing

  $opt .= " -pidfile $pidfile $dbopt -html -htmlfile $file -links";
  $opt .= " -support " if ($evidence) ;
  
  # Check for spangle jobs already running
  
  open(GREP,"ps -ef |grep \"spangle \" |");
  my $gr;
  
  while (<GREP>) {
      if ($_ !~ /grep/) {
	  $gr++ ;
      }
  }
  
  if ($gr > 4) {
      print("Spangle already running $gr processes.<br>\n");
      print(STDERR "Spangle already running $gr processes\n");
      exit(0);
  }

    print("<h2>Clone is $clone</h2>\n");

    #Jobs::fork_job($command,$pidfile,600);

    my $tmp_output = "/tmp/$clone.error";
    my $command =  Spangle::CGI::spangle() ." $opt $clone > $tmp_output 2>&1";
    #print qq{<pre>command = '$command'\n};
    print("Running spangle analysis. Please wait<br>\n");
    
    # Exit status of commands under CGI unreliable?
    system($command);
    
    if (1) {
        print("</PRE>\n");
    } else {
        open TMP_OUTPUT, $tmp_output
            or warn "Can't read '$tmp_output' : $!";
        my @out = <TMP_OUPUT>;
        close TMP_OUTPUT;
        print "Spangle output:\n", @out, "</PRE>\n";
    }
    unlink($tmp_output);

    chmod 0666, $file;

    if ($do_subseq == 1 && $subseq ne "") {
        $subseq =~ s/\"//g;

        my $subhtmlfile = $file;
        $subhtmlfile =~ s/\.html//;
        $subhtmlfile = $subhtmlfile . ".$subseq.html"; 

        open(HTML,"<$subhtmlfile");
        while (<HTML>) {
            print($_);
        }
        close(HTML);
    } else {
        print("Html file is <a href=\"$htmlfile\">here</a>\n");
    }
}  

sub modifyGene { 
    my ($file) = @_;
    my $action = $query->param('action');

    return unless ($action eq "Apply changes" || $action eq "Update form" || $action eq "Add remark");
		 
    my $path = $query->param('acepath');
    
    
    # Find the gene name (new or use the current one)
    my $genename;
    
    
    $genename = $query->param('genename')    if ($query->param('modify') eq "modify");
    $genename = $query->param('newCDSname')  if ($query->param('modify') eq "createCDS");
    $genename = $query->param('newmRNAname') if ($query->param('modify') eq "createmRNA");
    
    unless ($genename) {
	printf($file "ERROR: No gene name entered");
	return;
    }
    
    # Extract the clone name and check we aren't trying to
    # annotate a different clone
    my $clone = $genename;
    $clone =~ s/(.*?)\..*/$1/;
    
    my ($oldclone) = $query->param('genename') =~ /(.*?)\./;
    
    # $oldclone won't be defined if we are coming from an "ENSG\d+" object
    if ($oldclone and $oldclone ne $clone) {
	print($file "ERROR: Clone '$clone' does not match clone being annotated: '$oldclone'");
	return;
    }
    
    print($file "<center>\n");
    print($file "<h1>Clone name = $clone</h1>\n");
    
    my $do_coord;

    if ($query->param('action') eq "Update form" || $query->param('action') eq "Add remark") {
      $do_coord = 0;
    } else {
      $do_coord = 1;
    }

    my $gene = parseExons($genename,$do_coord);

    # Sort the exons by coordinate only if actually write to the database
    if ($action eq "Apply changes") {
      $gene->sort;
    }

    my $path = $query->param('acepath');
  
    my $acefile;
    my $tmp;
    my $ace;
    
    my $type;

    if ($query->param('modify') eq "createCDS") {
      $type = "supported_CDS";
    } elsif  ($query->param('modify') eq "createmRNA") {
      $type = "supported_mRNA";
    } else {
      $type = $query->param('subtype');
    }
    
    
    my $locus             = $query->param('locusname');
    my $start_not_found   = $query->param('start_not_found');
    my $CDS_start         = $query->param('CDS_start');
    my $CDS_end         = $query->param('CDS_end');
    my $end_not_found     = $query->param('end_not_found');
    my $pseudogene        = $query->param('pseudogene');
    $query->param('st_not_found') =~ /(\d)/;
    my $frame  = $1;
    
    
    $gene->locus          ($locus);
    $gene->start_not_found($start_not_found);
    $gene->end_not_found  ($end_not_found);
    $gene->pseudogene     ($pseudogene);
    $gene->CDS_start      ($CDS_start);
    $gene->CDS_end      ($CDS_end);
    $gene->frame ($frame);
    
    my @remarks;

    foreach my $p ($query->param) {
      if ($p =~ /remark(\d+)/) {
	$remarks[$1] = $query->param($p);
      }
    }

    foreach my $r (@remarks) {
      if ($r ne "") {
	$gene->addRemark($r);
      }
    }

    if ($query->param('action') eq "Add remark") {
      $gene->addRemark("");
    }
    
    if ($query->param('action') eq "Update form" || $query->param('action') eq "Add remark") {
      Spangle::CGI::print_gene_form($clone,$gene,$path,$display,\*STDOUT,$type,1);
      exit(1);
    }

 
    # Create an acefile for modified subsequences
    if ($path ne "") {
      if ($path !~ /\/$/) {
	$path .= "/";
      }
      
      ($acefile,$tmp) = Spangle::CGI::get_random_httpfile("${path}rawdata/support",".ace");
      
      open(ACE,">$acefile") || warn "Can't write to $acefile";
      $ace = \*ACE;
      
      (my $acepetal = $acefile) =~ s/.*\/(.*)/$1/;
      
#      printf($file "<p><font size=+2>Writing subsequence to <b>$acepetal</b></font><p>\n");
#      print("Choose 'Read .ace file' in xace to import data and then save before further analysis. <p>\n");
      
    } else {
      $ace = \*STDOUT;
    }
    
    
#    Spangle::CGI::printFooter($file);
    
    unless($acefile) {
	print($file "<PRE>\n");
    }
    
    writeAceSubsequence($clone,$gene,$type,$ace);	
    
    unless($acefile) {
	print($file "</PRE></CENTER>\n");
    }
    
    if ($acefile) {
      close($ace);
      print($file "Ace file is $acefile<br>\n");

      if ($display eq "") {
	print($file "<FONT COLOR=red>ERROR: no display set\n</FONT>\n");
      }

      my $command = "DISPLAY=$display;export DISPLAY; xremote  -remote \' pparse $acefile ; gif ; seqget $clone ; seqdisplay \'";
      my $out = system("$command");#  > /dev/null 2>&1");

      if ($out == 0) {
	print($file "<br>Ace database updated - select recalculate in Fmap and save to see changes\n");
      } else {
        print($file "<BR><PRE>COMMAND $command</PRE>\n",
            "xace -remote return value = $out\n",
	    "<FONT COLOR=red>ERROR updating ace database</FONT>\n",
            "Try typing <b><code>xhost intweb</code></b> in a terminal\n");
      }

    }

    Spangle::CGI::print_gene_form($clone,$gene,$path,$display,\*STDOUT,$type);

}

sub parseExons {
    my ($genename,$do_coord) = @_;

    my $gene = new Spangle::Subseq;

    my $strand  = $query->param('strand');

    $gene->strand($strand);

#    print("Strand is $strand\n");

    my @exons;
    my @newexons;

    # Read the existing exons from the html parameters
    foreach my $key ($query->param) {
	
	if ($key =~ /exon(\d+)/) {
	    my $num = $1;
	    
	    $exons[$num] = {};
	    $exons[$num]->{change}  = $query->param($key);
	    
	    $exons[$num]->{start}   = $query->param("exonstart$num");
	    $exons[$num]->{end}     = $query->param("exonend$num");
	}
    }


    
    # Adjust the list of exons according to the change params
    my $count = 0;
    foreach my $e (@exons) {


	if ($e->{change} eq "no change") {
	    push(@newexons,$e);
	    
	} elsif ($e->{change} eq "exon before") {
	    my $tmp = {};
	    $tmp->{change} = "";
	    
	    my $mid;
	    
	    if ($strand == 1) {

	      if ($do_coord) {
		if ($#newexons < 0) {
		  $mid = int($e->{start} - 100);
		} else {
		  $mid = int(($e->{start} + $newexons[$#newexons]->{end})/2);
		}
		
		$tmp->{start} = $mid - 20;
		$tmp->{end}   = $mid + 20;
	      } else {
		$tmp->{start} = 0;
		$tmp->{end}   = 0;
	      }
	      push(@newexons,$tmp,$e);
		
	    } elsif ($strand == -1) {
		print("strand = $strand $count\n");
		print("before " . $e->{start} . "\t" . $e->{end} . "\n");
		if ($do_coord) {
		  if ($e == $exons[0]) {
		    print("start exon\n");
		    $mid = $e->{end} + 60;
		  } else {
		    print("mid exon\n");
		    $mid = int(($exons[$count-1]->{end} + $e->{start})/2);
		  }
		  
		  $tmp->{start} = $mid - 20;
		  $tmp->{end}   = $mid + 20;
		} else {
		  $tmp->{start} = 0;
		  $tmp->{end}   = 0;
		  
		}
		print("New coords are " . $tmp->{start} . "  ". $tmp->{end} . "\n");
		push(@newexons,$tmp,$e);
	    }
	    
	} elsif ($e->{change} eq "exon after") {
	    my $tmp = {};
	    $tmp->{change} = "";
	    
	    if ($strand == 1) {
	      if ($do_coord) {
		$tmp->{start} = $e->{end} + 20;
		$tmp->{end}   = $e->{end} + 60;
	      } else {
		$tmp->{start} = 0;
		$tmp->{end}   = 0;

	      }
		push(@newexons,$e,$tmp);
		
	    } elsif ($strand == -1) {
		my $mid;
	       print("strand $strand\n");
		print("exon " . $e->{start} . "\t" . $e->{end} . "\n");
		if ($do_coord) {
		  if ($e == $exons[$#exons]) {
		    print("end exon\n");
		    $mid = $e->{start} - 60;
		  } else {
		    print("mid exon\n");
		    $mid = int(($e->{start} + $exons[$count+1]->{end})/2);
		  }
	  
		  $tmp->{start} = $mid - 20;
		  $tmp->{end}   = $mid + 20;
		} else {
		  $tmp->{start} = 0;
		  $tmp->{end}   = 0;
		  
		}
		print("New coords are " . $tmp->{start} . "  ". $tmp->{end} . "\n");
		push(@newexons,$e,$tmp);
	    }
	}
    $count++;
}

    # Now finally construct the gene

    $gene->source_tag($genename);
    $gene->strand    ($strand);

    foreach my $e (@newexons) {
	
	my $exon = new Bio::EnsEMBL::Exon;
	
	$exon->source_tag($genename);
	$exon->start     ($e->{start});
	$exon->end       ($e->{end});
        $exon->strand    ($strand);

	$gene->add_Exon  ($exon);
    }
    
    # sort the exons - forward for forward genes  - reverse for reverse genes

#    $gene->sort;
    my @tmp = $gene->each_Exon;
    return $gene;
}

sub writeAceSubsequence {
    my ($clone,$gene,$type,$file) = @_;

#    if ($gene->CDS_start == 1) {
#	$gene->CDS_start("");
#    }
    my $min = 10000000;
    my $max = -1;
    my $strand = $gene->strand;

    foreach my $ex ($gene->each_Exon) {
	if ($ex->start < $min) {$min = $ex->start;}
	if ($ex->start > $max) {$max = $ex->start;}
	if ($ex->end   < $min) {$min = $ex->end;}
	if ($ex->end   > $max) {$max = $ex->end;}
    }
    
    
    # Enter the subsequence in the clone object
    printf($file "\nSequence : $clone\n");

    # Delete any existing subsequence first
    printf($file "-D Subsequence "  . $gene->source_tag . "\n");

    if ($strand == 1) {
	printf($file "Subsequence " . $gene->source_tag . " $min $max\n");
    } elsif ($strand == -1) {
	printf($file "Subsequence " . $gene->source_tag . " $max $min\n");
    }

    # Now the subsequence object itself
    printf($file "\nSequence : " . $gene->source_tag . "\n");

    # Delete and replace exons
    printf($file "-D Source_Exons\n");
    printf($file "Source $clone\n");

    foreach my $ex ($gene->each_Exon) {
	if ($strand == 1) {
	    my $start = ($ex->start - $min+1);
	    my $end   = ($ex->end-$min+1);
	    if ($start > $end) {
		my $tmp = $start;
		$start = $end;
		$end   = $tmp;
	    }
	    printf($file "Source_Exons  " . $start . " " . $end   . "\n");
	} else {
	    my $start = ($max-$ex->start+1);
	    my $end   =  ($max-$ex->end+1);
	    if ($start > $end) {
		my $tmp = $start;
		$start = $end;
		$end   = $tmp;
	    }
	    printf($file "Source_Exons  " . $start     . " " . $end . "\n");
	}
    }
    
    #new implementation CDS int int : translate back to local (spliced mRNA) coordinates
    
    my $cdsstart = $gene->CDS_start;
    my $cdsend = $gene->CDS_end;
    my $cdsstartlocal = 0;
    my $cdsendlocal = 0;
    
    if ($cdsstart ne "1" && $cdsstart ne ""){  # if cds 1 or set but empty, cdsstartlocal = 1
        if ($strand == 1){
            foreach my $ex ($gene->each_Exon) {
                my $start = ($ex->start);
                my $end   = ($ex->end);
                if ( $start <= $cdsstart && $cdsstart <= $end ){
                    $cdsstartlocal=    $cdsstartlocal + $cdsstart- $start +1 #calculate new local start
                }elsif ($cdsstart >=  $end  ){
                    $cdsstartlocal=    $cdsstartlocal + $end - $start +1  #else accumulate exon lengths
                }
                if ( $start <= $cdsend && $cdsend <= $end ){
                    $cdsendlocal=    $cdsendlocal + $cdsend- $start +1
                }elsif ($cdsend >=  $end  ){
                    $cdsendlocal=  $cdsendlocal +  $end - $start +1
                }
            }   
        }elsif ($strand == -1){
            foreach my $ex ($gene->each_Exon) {
                my $start = ($ex->end);  # because for reverse strand genes the exon start and
                my $end   = ($ex->start);# end is flipped in Clone::subcoords2real
                if ( $start >= $cdsstart && $cdsstart  >= $end ){
                    $cdsstartlocal=    $cdsstartlocal - $cdsstart+ $start +1
                }elsif ($cdsstart <=  $end  ){
                    $cdsstartlocal=   $cdsstartlocal + $start - $end +1
                }
                if ( $start >= $cdsend && $cdsend >= $end ){
                    $cdsendlocal=    $cdsendlocal - $cdsend+ $start +1
                }elsif ($cdsend <=  $end  ){
                    $cdsendlocal=   $cdsendlocal + $start - $end +1
                }
            }   
        }
#    }elsif($cdsend eq ""){  # start =1, end has non-zero value
#        if ($strand == 1){
#            foreach my $ex ($gene->each_Exon) {
#                if ( $start <= $cdsend && $cdsend <= $end ){
#                $cdsendlocal=    $cdsendlocal + $cdsend- $start +1
#                }elsif ($cdsend >=  $end  ){
#                $cdsendlocal=  $cdsendlocal +  $end - $start +1
#                } 
#            }
#        }elsif ($strand == -1){
#            foreach my $ex ($gene->each_Exon) {
#                if ( $start >= $cdsend && $cdsend >= $end ){
#                $cdsendlocal=    $cdsendlocal - $cdsend+ $start +1
#                }elsif ($cdsend <=  $end  ){
 #               $cdsendlocal=   $cdsendlocal + $start - $end +1
 #               }
 #           }
 #       }        
 #       $cdsstartlocal = 1;
    }else{
        foreach my $ex ($gene->each_Exon) {
            my $start = ($ex->start);
            my $end   = ($ex->end);
            $cdsendlocal = $cdsendlocal + abs ($end - $start) +1
        }
        $cdsstartlocal = 1;
    }

    printf($file "CDS " . $cdsstartlocal . " ". $cdsendlocal . "\n");
    
    

    # Type of gene
    if ($gene->pseudogene == 1) {
    	printf($file "Method Pseudogene\n");  
        printf($file "Properties Pseudogene\n");  
    } elsif ($type =~ /supported/) {
	printf($file "Method $type\n");
    } else 
        {$type =$query->param('subtype');
    }

    # Delete and replace Locus if necessary
    if ($gene->locus) {
	printf($file "-D Locus\n");
	printf($file "Locus " . $gene->locus . "\n");
    }
    
    # Now tags to say what type of gene am I
    printf($file "Properties Predicted_gene\n");

    my $cds;

    if ($gene->CDS_start == 1) {
	$cds = "";
    }

    if ($type eq "supported_mRNA") {
      printf($file "-D CDS\n");
      printf($file "processed_mRNA\n");
    } elsif ($type eq "supported_CDS") {
      printf($file "-D CDS\n");
      printf($file "CDS " . $cds . "\n");
    }


    # Delete and replace start_not_found/end_not_found
    if ($gene->start_not_found == 1) {
	printf($file "-D start_not_found\n");
	if ($gene->frame == 1) {
            printf($file "start_not_found\n");
        }else{
            printf($file "start_not_found  " . $gene->frame . "\n");
        }   
    } else {
	printf($file "-D start_not_found\n");
    }

    if ($gene->end_not_found == 1) {
	printf($file "end_not_found\n");
    } else {
	printf($file "-D end_not_found\n");
    }
    
    # Delete and replace the remark

    printf($file "-D Remark\n");

    my @remark = $gene->eachRemark;

    foreach my $r (@remark) {
      printf($file "Remark \"$r\"\n");
    }



    # Finally create the locus entry
    if ($gene->locus) {
	printf($file "\nLocus : ". $gene->locus . "\n");
	printf($file "Positive_sequence " . $gene->source_tag . "\n\n");
    }

    
    # Finally finally some polyA features
    
    my @pAsites;
    my @pAsignals;
    
    foreach my $key ($query->param) {
         #before bothering to make a hash etc, check whether there is any data
        if (($key =~ /polyA_site(\d+)/) && ($query->param("polyA_site_start$1"))){
            my $num = $1;
            $pAsites[$num] = $query->param("polyA_site_start$num");
        }
    }
    
    foreach my $key ($query->param) {
        if (($key =~ /polyA_signal(\d+)/) && ($query->param("polyA_signal_start$1"))){
            my $num = $1;
            $pAsignals[$num] = $query->param("polyA_signal_start$num"); 
        }
    }
    
    
    print($file "\nSequence \"$clone\"\n");
    
    foreach my $pA (@pAsites){
        unless ($pA == ""){
            if ($strand == 1) {
                print($file "Feature polyA_site " . ($pA -1). " " . $pA . " .5 polyA_site\n");
            }else{
                print($file "Feature polyA_site " . ($pA +1). " " . $pA . " .5 polyA_site\n");
            }
        }
    }
    
    
    foreach my $pA (@pAsignals){
        unless ($pA == ""){
            if ($strand == 1) {
                print($file "Feature polyA_signal " . $pA . " " . ($pA +5) . " .5 polyA_signal\n");
            }else{
                print($file "Feature polyA_signal " . $pA . " " . ($pA -5) . " .5 polyA_signal\n");            
            }
        }
    }
    

}
