### Bio::EnsEMBL::Ace::DataFactory

package Bio::EnsEMBL::Ace::DataFactory;

use strict;
use Carp;

#use Bio::EnsEMBL::Ace::Filter::Repeatmasker;
#use Bio::EnsEMBL::Ace::Filter::CpG;
#use Bio::EnsEMBL::Ace::Filter::DNA;
#use Bio::EnsEMBL::Ace::Filter::TRF;
#use Bio::EnsEMBL::Ace::Filter::Gene::Halfwise;
#use Bio::EnsEMBL::Ace::Filter::Gene;
#use Bio::EnsEMBL::Ace::Filter::Gene::Predicted;
#use Bio::EnsEMBL::Ace::Filter::Similarity;
#use Bio::EnsEMBL::Ace::Filter::Similarity::DnaSimilarity;
#use Bio::EnsEMBL::Ace::Filter::Similarity::ProteinSimilarity;
#use Bio::EnsEMBL::Ace::Filter::SimpleFeature;


sub new {
    my ($class) = @_;

    return bless {}, $class;
}

sub file_handle {
    my( $self, $fh ) = @_;
    
    if ($fh) {
        $self->{'_fh'} = $fh;
    }
    return $self->{'_fh'};
}

sub drop_file_handle {
    my $self = shift;
    
    $self->{'_fh'} = undef;
}

sub ace_data_from_slice {
    my ($self, $slice) = @_ ; 
        
    my @filters = $self->get_all_AceFilters;
    warn "factory has had no filters added!" unless @filters;
    
    if (my $fh = $self->file_handle) {
        foreach my $filt (@filters) {
            if ($filt->can('file_handle')) {
                $filt->file_handle($fh);
                $filt->ace_data($slice);
                $filt->drop_file_handle;
            } else {
                print $fh $filt->ace_data($slice);
            }
        }
    } else {
        my $ace_data = '';
        foreach my $filt (@filters) {
            $ace_data .= $filt->ace_data($slice);
        }
        return $ace_data;    
    } 
}

sub add_AceFilter {
    my( $self, $filter ) = @_;

    my $af_list = $self->{'_ace_filter_list'} ||= [];
    push @$af_list, $filter;
}

sub get_all_AceFilters {
    my( $self ) = @_;

    if (my $af_list = $self->{'_ace_filter_list'}) {
        return @$af_list;
    } else {
        return;
    }
}




1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::DataFactory

=head1 AUTHOR

Colin Kingswood<email> ck2@sanger.ac.uk
email: No such file or directory.


