
package Bio::EnsEMBL::Ace::Filter::Cigar_ace_parser;

## module currently takes in a cigar string and outputs three arrays which are the acedb 'Align' coords 
## also inputs start coordinates for the sequence and target if available, and outputs correspondonding  
## cooordinates (sub all_coords includes the start and end -which are not used yet, but maybe someone
## will want to in the future)

use strict;
use Carp;

sub align_coords {
    my ($cigar, $query_start, $query_end, $target_start, $strand, $is_protein) = @_;


    # if we are dealing with a protein, coords need multiplied by 3
    my $protein_multiplier = $is_protein ? 3 : 1;

    # split cigar string into list of number and letter pairs (number is optional, if not present, it is defaulted to 1)
    #my @cigar_elements = split (/(\D)/m, $cigar);
    my( @cigar_elements );
    while ($cigar =~ /(\d+)?(\D)/g) {
        push(@cigar_elements, [$1 || 1, $2]);
    }

    # if on negative strand , reverse the cigar string  (so we work through it backwards)
    # start position should be the end coord (length of the cigar line + start coord -1) 
    # note: AceDb needs the target coords starting at the small end and working bigger
    my $start_pos = $query_start;
    if ($strand == -1) {
        $start_pos = $query_end;

        ### I think we should really pass in the query and subject strands, and
        ### only reverse the cigar string if the query (contig) strand is minus?
        ### Related to est2genome contig_strand always forwards issue.
        @cigar_elements = reverse @cigar_elements unless $is_protein;
    }

    my (@query_coords , @target_coords , @extra_coords);   # for the output
    push (@query_coords, $start_pos ) ; # put start coord of the query sequence here
    push (@target_coords , $target_start);# and the start of the target here
    push (@extra_coords ,  '');

    my $previous_letter = '';
    my $previous_number = '' ;
    
    # We don't need the end of the cigar string for acedb to be able to reconstruct the alignment
    pop(@cigar_elements);
    
    foreach my $pair (@cigar_elements) {


        my $number = $pair->[0];
        my $letter = $pair->[1];
#            print STDERR "$number  $letter \n";         


        # for non 'match' elements of the array, add the $number to the existing last
        # element as there should be  a previous match to this.
        # note that introns ('N') are essentially large inserts so are treated the same
        if (($letter eq  'I') or ($letter eq 'N')) { 
            $query_coords[$#query_coords] += ($number * $strand);
            if ($previous_letter eq 'D'){
                # if an insertion immedieately follows a deletion , use the third column for length of match
                $extra_coords[$#extra_coords - 1] = $previous_number;         
            }
            $previous_letter = 'I';

        }
        elsif ($letter eq 'D') {
            $target_coords[$#target_coords] += int($number / $protein_multiplier ) ;
            if ($previous_letter eq 'I') {
                $extra_coords[$#extra_coords - 1] = $previous_number;
            }
            $previous_letter = 'D';    

        }
        elsif ($letter eq 'M') {
            # for each match , a new element is added to each array
            # and the '$number' is added to the previous value
            push (@query_coords , ( ($number * $strand)  + $query_coords[$#query_coords]));
            push (@target_coords, int( ($number / $protein_multiplier )  + $target_coords[$#target_coords]) );
            push (@extra_coords , '');
            $previous_letter = '';
            $previous_number = $number;
        }
        else {
            confess "Unkown letter '$letter' found in cigar string '$cigar'";
        }
    }
    
    return (\@query_coords, \@target_coords, \@extra_coords);
}



1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::Cigar_ace_parser

=head1 AUTHOR

Colin Kingswood <email> ck2@sanger.ac.uk
