
### Bio::EnsEMBL::Ace::Filter::DNA

package Bio::EnsEMBL::Ace::Filter::DNA;

use strict;
use Carp;
use Bio::SeqIO;
use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');

## takes in a slice and creates AceDB data from it.

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;
    return $self;
}

sub ace_data {
    my ( $self, $slice ) = @_;
    

    my $ace;
    
    # slice is split up into the individual contigs that make it up
    my $tp = $slice->get_tiling_path;
#    my $slice_name = $slice->name;
    foreach my $tile (@$tp)
    {   
        my $id = $tile->component_Seq->name;
        my $dna_str = $tile->component_Seq->seq;
        $ace .= qq{\nSequence : "$id"\nGenomic_canonical\n}
             . qq{DNA "$id"\n}

             . qq{\nDNA : "$id"\n};
    
        my $tile_length = length($dna_str);

        while ($dna_str =~ /(.{1,60})/g) {
               $ace .= "$1\n";
        }
                  
       $ace .= $self->non_golden_features($tile, $tile_length); 
    }
    
    $ace .= $self->clone_left_right($slice);
    
    return $ace ;   
}

sub clone_left_right {
    my( $self, $slice ) = @_;
    
    my $name      = $slice->display_id;
    my $path      = $slice->get_tiling_path;

    return '' unless @$path;

    my $ace = qq{\nSequence : "$name"\n};
    my $clone_ace = '';

    # Clone_left_end and Clone_right_end
    foreach my $tile (@$path) {

        if (my $clone = $tile->component_Seq->clone) {
            my $start    = $tile->assembled_start;
            my $end      = $tile->assembled_end;
            my $seq_name = $tile->component_Seq->name;
            my $id  = $clone->id;
            #my $acc = $clone->embl_id;
            #my $sv  = $clone->embl_version;
            $ace .= qq{Clone_left_end "$id" $start\n}
                 . qq{Clone_right_end "$id" $end\n};
            
            $clone_ace .= qq{\nClone : "$id"\nSequence "$seq_name"\n};
            #my $contig_start = $tile->component_start;
            #my $contig_end   = $tile->component_end;

            #if ($tile->component_ori == 1) {
            #    $clone_ace .= qq{AGP_Fragment "$name" $contig_start $contig_end Align $contig_start $start\n};
            #} else {
            #    # Clone in reverse orientaton in AGP is indicated
            #    # to acedb by first coordinate > second
            #    $clone_ace .= qq{AGP_Fragment "$name" $contig_end $contig_start Align $contig_start $end\n};
            #}
        }

    }
    
    return $ace . $clone_ace;
}

sub non_golden_features{
    
    my ($self , $tile , $overall_length) = @_ ;
    
    my $id = $tile->component_Seq->name;
    my $golden_start = $tile->component_start; # start coordinate of section of contig that is on the gloden path
    my $golden_end = $tile->component_end;  #end coordinate of section that is on the gloden path
    
    my $golden_length = $tile->assembled_end - $tile->assembled_start ;
        
    my $feats = '';
    
    
    
    # if the start of this contig is not on the golden path,then output this as a feature
    if ($tile->component_ori != $tile->component_start) {
        $golden_start--;
        $feats .= qq{Feature "NonGolden" 1 $golden_start 100 "Region of $id not on golden path"\n};
    }
    
    #if end of this contig is not on the golden path then output this as a feature
    if ( $golden_length != ($overall_length - $tile->component_start)) {
        $golden_end++;
        $feats .= qq{Feature "NonGolden" $overall_length $golden_end 100 "Region of $id not on golden path"\n};
    }
    
    if ($feats) {
        return qq{\nSequence : "$id"\n} . $feats;
    } else {
        return "";
    }
    
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::DNA

=head1 AUTHOR

Stephen Keenan B<email> keenan@sanger.ac.uk

