
### Bio::EnsEMBL::Ace::Filter::Gene::Halfwise

package Bio::EnsEMBL::Ace::Filter::Gene::Halfwise;

use strict;
use Carp;
use Data::Dumper;
use Bio::EnsEMBL::Ace::Filter::Gene;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter::Gene');

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;

#    $self->url_string('http\\:\\/\\/www.sanger.ac.uk\\/cgi-bin\\/Pfam\\/getacc?%s' );
    
    return $self;
}

sub make_transcript_ace_name {
    my( $self, $transcript ) = @_;
    
    my $pfam_acc = $transcript->temporary_id;
    return $pfam_acc . '.' . ++$self->{'_domain_int'}{$pfam_acc};
}


#----------------------------------------------------------------------------------------------
sub get_transcripts{
    my ( $self, $slice  ) = @_;

    my $transcript_list;
    my $genes = $slice->get_all_Genes('Halfwise');
    
    foreach my $gene (@$genes) {
        my( $xref, $gene_name );
        foreach my $xr (@{$gene->get_all_DBEntries}) {
            if ($xr->primary_id =~ /^PF/) {
                $xref = $xr;
                $gene_name = $xr->primary_id;
            }
        }
        confess "Can't find Pfam DBEntry" unless $xref;

        $gene_name =~ s/\.\d+$//;
        
        foreach my $t (@{$gene->get_all_Transcripts}) {
            $t->display_xref($xref);
            $t->temporary_id($gene_name);
            push(@$transcript_list, $t);
        }
    } 
    return $transcript_list;
} 
    
##-----------------------------------------------------------------------------------------
sub transcript_exons_in_slice{
    my ($self,  $slice, $transcript) = @_;
    
    unless ($slice and $transcript) {
        confess "need both slice and transcript";
    }
    
    my $contig_id = $slice->name;
    my $tempvar = $transcript ->get_all_Exons();
    my @exons_in_transcript = @$tempvar;
    my ($snf , $enf );
    
    for (my $i = 0; $i < @exons_in_transcript;) {
        my $exon = $exons_in_transcript[$i];
        my $ex_con_id = $exon->contig->name;
        if ($ex_con_id eq $contig_id) {
            $i++;
        } else {
            #warn "exon '$ex_con_id' is not in VirtualContig\n";
            splice(@exons_in_transcript, $i, 1);
            if ($i == 0) {
                $snf = 1;
            } else {
                $enf = 1;
            }
        }
    }   
    return ($snf , $enf , @exons_in_transcript)
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::Gene::Halfwise

=head1 AUTHOR

James Gilbert B<email> jgrg@sanger.ac.uk

