
### Bio::EnsEMBL::Ace::Filter::Repeatmasker

package Bio::EnsEMBL::Ace::Filter::Repeatmasker;

use strict;
use Carp;
use strict;
use Bio::SeqIO;
use Data::Dumper;

use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');

sub new {
    my ($class) = @_;

    my $self = bless {}, $class;
    return $self;
}


# updated to work from a slice rather than a clone
sub ace_data {     
    my ( $self, $slice ) = @_;

    my $ana_obj = $self->analysis_object
        or confess "analysis_object not set";

    my $ana_id = $ana_obj->dbID;
    
    
    my $db_obj = $slice->adaptor->db();
        
    my $repeat_ace; # to store output
    my $count = 0;  # to count if any lines are created from query
    my $id = $slice->name;  
    my $tp = $slice->get_tiling_path();
    foreach my $tile ( @$tp)    
    {   
        my $seq = $tile->component_Seq;
        my $contig_id = $seq->dbID;
        my $contig_name = $tile->component_Seq->name;
        
        #use list of contigs in next database  query where we get the desired information.
        my $sql = qq{
            SELECT r.contig_start
              , r.contig_end
              , r.score
              , r.contig_strand
              , r.repeat_start
              , r.repeat_end
              , rc.repeat_name
              , rc.repeat_class
            FROM repeat_feature r ,
                repeat_consensus rc
           WHERE r.repeat_consensus_id = rc.repeat_consensus_id
              AND  contig_id  = $contig_id
              AND analysis_id = $ana_id
            ORDER BY rc.repeat_consensus_id
            };
        #warn "query = '$sql'";
        my $sth = $db_obj->prepare($sql);
        $sth->execute;
        
 
        my ( $ctg_start,  $ctg_end,  $score,  $strand,  $rpt_start,  $rpt_end,  $rpt_cons_id ,$class );
        $sth->bind_columns(
            \$ctg_start, \$ctg_end, \$score, \$strand, \$rpt_start, \$rpt_end, \$rpt_cons_id , \$class );
 
        
        

        
        my $temp_repeat_ace =''; # this is used, to store the output for one patrticular contig. (so a sequence name is not added when no repeats)
        while ($sth->fetch) {
            $count++;
            my $feature_name = 'RepeatMasker';

            if ($class =~ m/LINE/)
            {
                $feature_name = 'RepeatMasker_LINE';
            }
            if ($class =~ m/SINE/)
            {
                $feature_name = 'RepeatMasker_SINE' 
            }
            if ($strand == 1) {
                $temp_repeat_ace .=
                    qq{Motif_homol "$rpt_cons_id" "$feature_name" $score $ctg_start $ctg_end $rpt_start $rpt_end\n};
            }
            else {
                $temp_repeat_ace .=
                    qq{Motif_homol "$rpt_cons_id" "$feature_name" $score $ctg_end $ctg_start $rpt_start $rpt_end\n};
            }
        }
        
        if ($temp_repeat_ace)
        {   #add heading and data only when there is data persent  
            $repeat_ace .= qq{\nSequence "$contig_name"\n};
            $repeat_ace .= $temp_repeat_ace;
        }
    }
    if ($count) {

        return $repeat_ace;
    } else {
        return '';
    }
}

sub repeatmasker_home {
    my( $self, $home ) = @_;
    
    if ($home) {
        confess "Not a directory: '$home'"
            unless -d $home;
        $self->{'_repeatmasker_home'} = $home;
    }
    return $self->{'_repeatmasker_home'};
}

{
    my( $repeats );

    sub get_repeat_class_hash {
        my( $self ) = @_;

        unless ($repeats) {
            $repeats = {};

	    # potential alternative source, if LACE_LOCAL set
	    if($ENV{"LACE_LOCAL"}){
		my $rep_header_file=$ENV{"LACE_LOCAL"}."/repclass.lis";
		if(-e $rep_header_file){
		    local(*REPCLASS);
		    open(REPCLASS, $rep_header_file) or confess "Cannot open $rep_header_file: $!";
		    while (<REPCLASS>) {
			my($rep,$class)=split(/\t/);
			$repeats->{$rep} = $class;
		    }
		    close(REPCLASS);
		    print "INFO: used FILE $rep_header_file (".scalar(keys %$repeats).")\n";
		    return $repeats;
		}
	    }

            my $rep_home = $self->repeatmasker_home
                or confess "repeatmasker_home not set";
            my $rep_lib_dir = "$rep_home/Libraries";
            local(*REPLIB, *REPFILE);
            opendir REPLIB, $rep_lib_dir or confess "Can't opendir '$rep_lib_dir' : $!";
            

            foreach my $rep_file (map "$rep_lib_dir/$_", sort grep /\.lib$/, readdir REPLIB) {

                open REPFILE, $rep_file
                    or confess "Can't open '$rep_file' : $!";
                while (<REPFILE>) {
                    if (/^>(.*)#(\w+)/) {
                        my( $exist_class );
                        if ($exist_class = $repeats->{$1} and $exist_class ne $2) {
                             #warn qq{Repeat '$1' already exists with class '$exist_class'\n  but has class '$2' in '$rep_file'\n} ;
                        } else {
                            $repeats->{$1} = $2;
                        }
                    }
                }
                close REPFILE;
            }
            closedir REPLIB;
	    print "INFO: used DIR $rep_lib_dir (".scalar(keys %$repeats).")\n";
        }
        return $repeats;
    }
}

1;

__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::Repeatmasker

=head1 AUTHOR

Stephen Keenan B<email> keenan@sanger.ac.uk

