
### Bio::EnsEMBL::Ace::Filter::SimpleFeature

package Bio::EnsEMBL::Ace::Filter::SimpleFeature;


use strict;
use Carp;

use Bio::SeqIO;
use Bio::EnsEMBL::Ace::Filter;

our @ISA = ('Bio::EnsEMBL::Ace::Filter');

sub new {
    my ($class) = @_;

    my $feat = bless {}, $class;
    return $feat;
}

sub _ace_sprintf{
    warn "_ace_sprintf not implemented";
    return q`Feature %s ` . "\n";
}

sub method_tag{
    my ($self, $meth) = @_;
    $self->{'_method_tag'} = $meth if $meth;
    return $self->{'_method_tag'} || '';
}
sub _sprintf_array_from_cols{
    # automatically called with contig_start, contig_end, strand, score, display_label
    shift; # remove reference to "self"
    return @_;
}
sub ace_data {
    my ($self , $slice) = @_;
    
    my $ana_obj = $self->analysis_object || confess "analysis_object not set";
    my $ana_id  = $ana_obj->dbID;
    my $db_obj  = $slice->adaptor->db;
    my $tp      = $slice->get_tiling_path;
    my $ace_data;
    my $ace_sprintf = q`Feature %s ` . $self->_ace_sprintf() . "\n";
    my $method      = $self->method_tag();

    foreach my $tile (@$tp){
        my $raw_contig_id = $tile->component_Seq->name  ;
        my $seqname = "\nSequence : ".$tile->component_Seq->name."\n";        
        my $temp_ace_data = '';

        #warn "looking for ana id $ana_id \n";
        
        my $sth = $db_obj->prepare(qq{
        SELECT contig_start
          , contig_end
          , contig_strand
          , score
          , display_label
        FROM simple_feature , contig
        WHERE simple_feature.contig_id = contig.contig_id
        AND contig.name = '$raw_contig_id'  
        AND analysis_id = $ana_id

        });
        $sth->execute;
        my                ( $start,  $end,  $strand,  $score,   $display_label);
        $sth->bind_columns(\$start, \$end, \$strand, \$score , \$display_label);
        
        while ($sth->fetch){
            my $this_ace = sprintf($ace_sprintf,
                                   $method, 
                                   $self->_sprintf_array_from_cols($start, $end, $score, $display_label)
                                   );
            $temp_ace_data .= $this_ace;
        }
        if ($temp_ace_data){
            $ace_data .= $seqname ;
            $ace_data .= $temp_ace_data;
        }
    }    
    return $ace_data ? $ace_data : '';
}

1;

__END__


=head1 NAME - Bio::EnsEMBL::Ace::Filter::SimpleFeature

=head1 AUTHOR

Roy Storey B<email> rds@sanger.ac.uk

