
### Bio::EnsEMBL::Ace::Filter::TRF

package Bio::EnsEMBL::Ace::Filter::TRF;

use strict;
use Carp;
use Bio::EnsEMBL::Ace::Filter;
use vars '@ISA';
@ISA = ('Bio::EnsEMBL::Ace::Filter');


sub ace_data{
    my ($self, $slice) = @_;
    
    my $repeat_ace = '';
    my $db_obj = $slice->adaptor->db;   
    my $ana_obj = $self->analysis_object
        or confess 'analysis_object not set';
    my $analysis_id = $ana_obj->dbID;
    my $feature_name = $ana_obj->logic_name;

    
    my $tp = $slice->get_tiling_path;
    foreach my $tile (@$tp)
    {
        my $name = $tile->component_Seq->name;
        my $contig_id = $tile->component_Seq->dbID;
        
        my $sth = $db_obj->prepare(qq{
            SELECT contig_start
                , contig_end
                , score
                , rc.repeat_consensus
            FROM  repeat_feature rf, repeat_consensus rc 
            WHERE rc.repeat_consensus_id = rf.repeat_consensus_id 
            AND rf.contig_id = ?
            AND rf.analysis_id  =  ?
            });
            
        $sth->execute($contig_id , $analysis_id  );
            
        my ($start, $end , $score, $repeat);
        $sth->bind_columns(\$start,\$end, \$score, \$repeat);
        
        
        # format output into a temporary string
        my $temp_repeat_ace = '';
        while ($sth->fetch)
        {   
            my $mer_length = length($repeat);
            my $rep_length = $end - $start + 1;
            
            
            $temp_repeat_ace .= sprintf(
                qq{Feature "%s" %d %d %f "%.1f copies %d mer %s"\n},
                $feature_name, $start , $end, $score, ($rep_length / $mer_length) , $mer_length , $repeat 
                );
        }
        
        #if there is a value in the temporary string, 
        #then add a contig_name to the output as well as the data in the temp string
        if ($temp_repeat_ace)
        {   $repeat_ace .=  qq{\nSequence "$name"\n};
            $repeat_ace .= $temp_repeat_ace;
        }
        
        
    }    
    
    if ($repeat_ace)
    {
        return $repeat_ace;
    }
    else
    {   return '';
    }
}

##sub ace_data_slice_coords {
##    # not used but I have written it and it may be useful in the future;
##    # does the same as ace_data, except that the slice is not split into contigs 
##    # and the output is in slice coords.
##    
##    
##    my ( $self, $slice  ) = @_;
##    my $raw_contig_id;
##    my $contig_id;
##
    ##my $name = $slice->name;
    ##
    ##
    ##my $ana_obj = $self->analysis_object
    ##    or confess "analysis_object not set";

    ##
    ##my $count = 0;
    ##my $repeat_ace;
    ##
    ##my $rpts =$slice -> get_all_RepeatFeatures('TRF');
    ##my ( $start,  $end,  $score,  $rpt_cons );
    ##
    ##
    ##$repeat_ace = $repeat_ace . qq{\nSequence "$name"\n};
    ##foreach my $rf( @$rpts )
    ##{   
    ##    $count ++;
    ##    $rf->hstart;
    ##    $rf->hend;
    ##    $start = $rf->start + 100 ;
    ##    $end = $rf->end + 100 ;
    ##    $score =$rf->score;
    ##    $rpt_cons = $rf->repeat_consensus->repeat_consensus;
    ##    
    ##    my $mer_length = length($rpt_cons);
    ##    my $rep_length = $end - $start + 1;
    ##    
    ##    $repeat_ace .= sprintf(
    ##             qq{Feature "%s" %d %d %f "%.1f copies %d mer %s"\n},
    ##             $feature_name, $start, $end, $score, ($rep_length / $mer_length), $mer_length, $rpt_cons
    ##             );

##    } 
## 
## 
##    if ($count) {
##        return $repeat_ace;
##    } else {
##        return '';
##    }
##}   
1;


__END__

=head1 NAME - Bio::EnsEMBL::Ace::Filter::TRF

=head1 AUTHOR

James Gilbert B<email> jgrg@sanger.ac.uk

