#!/usr/local/bin/perl

use strict;
use Getopt::Long;

use Bio::Otter::DBSQL::DBAdaptor;


my $host    = 'humsrv1';
my $user    = 'ensro';
my $pass    = '';
my $dbname  = 'otter_human';
my $port    = 3306;

my $chr      = '14';
my $chrstart = 1;
my $chrend   = 300000000;
my $path     = 'GENOSCOPE';
my $opt_h;

my $filter_gd;
my @gene_names;
my $opt_t;

&GetOptions( 'host:s'      => \$host,
             'user:s'      => \$user,
             'pass:s'      => \$pass,
             'port:s'      => \$port,
             'dbname:s'    => \$dbname,
             'chr:s'       => \$chr,
             'chrstart:n'  => \$chrstart,
             'chrend:n'    => \$chrend,
             'path:s'      => \$path,
	     'filter_gd'   => \$filter_gd,
	     't'           => \$opt_t,
	     'h'           => \$opt_h,
            );

my %gene_names;
if (scalar(@gene_names)) {
  @gene_names=split (/,/, join (',', @gene_names));
  %gene_names = map {$_,1} @gene_names;
}

if($opt_h){
    &help;
}

sub help {
    print <<ENDHELP;
check_annotation.pl
  -host      char   host ($host)
  -user      char   user ($user)
  -port      num    port ($port)
  -dbname    char   dbname ($dbname)
  -chr       char   chr ($chr)
  -chrstart  num    start ($chrstart)
  -chrend    num    end ($chrend)
  -path      char   path ($path)
  -filter_gd        filter out GD genes

  -h                for help
ENDHELP

    exit 0;
}

my $sdb = new Bio::Otter::DBSQL::DBAdaptor(-host => $host,
                                           -user => $user,
                                           -pass => $pass,
                                           -port => $port,
                                           -dbname => $dbname);

$sdb->assembly_type($path);

my $sgp = $sdb->get_SliceAdaptor;
my $aga = $sdb->get_GeneAdaptor;
my $ega = $sdb->get_ExonAdaptor;

my $vcontig = $sgp->fetch_by_chr_start_end($chr,$chrstart,$chrend);
print "Fetched slice\n";

my $genes = $aga->fetch_by_Slice($vcontig);
print "Fetched ".scalar(@$genes)." genes\n";

my $exons = $ega->fetch_by_Slice($vcontig);
print "Fetched ".scalar(@$exons)." exons\n";

my %genehash;
my $ngd=0;
my $nobs=0;
my $nskipped=0;
foreach my $gene (@$genes) {
  my $gsi=$gene->stable_id;
  if(scalar(@gene_names)){
    next unless $gene_names{$gsi};
  }
  if($filter_gd){
    my $name=$gene->gene_info->name->name;
    if($name=~/\.GD$/ || $name=~/^GD:/){
      print "GD gene $gsi $name was ignored\n";
      $ngd++;
      next;
    }
  }
  $genehash{$gsi} = $gene;
}

print "$ngd GD genes removed\n";
print scalar(keys %genehash)." genes to check\n";
exit 0 if $opt_t;



